<?php

namespace JimsCustomerBooking\Admin\BackendPages;

use JimsCustomerBooking\Template;
use JimsCustomerBooking\Helpers\FSHelper;

if (!class_exists('JimsCustomerBooking\Admin\BackendPages\BackendPage')) {
    abstract class BackendPage
    {
        protected string $name = 'backend_page';
        protected string $title = '';
        protected string $sub_title = '';
        protected string $menu_title = '';
        protected string $capability = 'manage_options';
        protected string $parent_slug = '';
        protected string $menu_slug = '';
        protected string $icon_path = '';
        protected string $menu_icon = 'dashicons-screenoptions';
        protected int $position = 30;
        protected string $template_name = 'default_backend_page';
        protected bool $hide_menu_item = false;
        protected $hook = '';

        abstract public function getContent(): void;

        public function __construct()
        {
            add_action('admin_menu', [$this, 'setup']);

            if ($this->hide_menu_item) {
                add_action('admin_head', [$this, 'hideMenuItem']);
            }
        }

        public function setup(): void
        {
            if (!empty($this->parent_slug)) {
                $this->setupSubMenuPage();
            } else {
                $this->setupMenuPage();
            }
        }

        public function getName(): string
        {
            return $this->name;
        }

        public function getTitle(): string
        {
            return !empty($this->title)
                ? $this->title
                : preg_replace('#[-_]#', ' ', $this->name);
        }

        public function getMenuSlug($only_property = false): string
        {
            if ($only_property) {
                return $this->menu_slug;
            }

            if (empty($this->menu_slug)) {
                return "{$this->name}/{$this->name}.php";
            }

            return $this->menu_slug;
        }

        public function getHook(): string
        {
            return $this->hook;
        }

        public function hideMenuItem(): void
        {
            global $submenu;

            if (isset($submenu[$this->parent_slug])) {
                foreach ($submenu[$this->parent_slug] as $key => $menu_item) {
                    if ($menu_item[2] == $this->getMenuSlug(true)) {
                        unset($submenu[$this->parent_slug][$key]);
                    }
                }
            }
        }

        protected function getTemplate(): Template
        {
            return new Template($this->template_name);
        }

        protected function getMenuIcon(): string
        {
            return empty($this->icon_path)
                ? FSHelper::url('build/img/graphics/jims20x20.png')
                : $this->menu_icon;
        }

        protected function getMenuTitle(): string
        {
            return empty($this->menu_title) ? ucwords($this->getTitle()) : ucwords($this->getTitle());
        }

        private function setupMenuPage(): bool
        {
            $this->hook = \add_menu_page(
                $this->getTitle(),
                $this->getMenuTitle(),
                $this->capability,
                $this->getMenuSlug(),
                [$this, 'getContent'],
                $this->getMenuIcon(),
                $this->position
            );

            return \strlen($this->hook) > 0;
        }

        private function setupSubMenuPage(): bool
        {
            if (is_string($this->parent_slug)) {
                $this->hook = \add_submenu_page(
                    $this->parent_slug,
                    $this->getTitle(),
                    $this->getMenuTitle(),
                    $this->capability,
                    $this->getMenuSlug(),
                    [$this, 'getContent'],
                    $this->position
                );
            }

            return \strlen($this->hook) > 0;
        }
    }
}
