<?php

namespace JimsCustomerBooking\Admin\BackendPages;

use JimsCustomerBooking\Helpers\OptionHelper;
use JimsCustomerBooking\Helpers\SettingsHelper;
use JimsCustomerBooking\Helpers\StringHelper;
use JimsCustomerBooking\Services\DivisionService;

if (!class_exists('JimsCustomerBooking\Admin\BackendPages\ShortCodeGeneratorPage')) {
    class ShortCodeGeneratorPage extends BackendPage
    {
        use SettingsTrait;

        protected string $name = 'Shortcode Generator';
        protected string $parent_slug = 'wpcb_settings.php';
        protected string $menu_slug = 'cb_shortcode_generator.php';
        protected string $template_name = 'shortcode.generator';
        protected bool $hide_menu_item = true;

        public function getContent(): void
        {
            $options = OptionHelper::getEmbeddedFormOptions();

            $this->getTemplate()->render([
                'title' => 'Shortcode Generator',
                'fields' => $options->getProperties(),
                'native_fields' => OptionHelper::get(NativeFormSettingsPage::OPTION_NAME, []),
                'JimsOptions' => $options,
                'pages' => SettingsHelper::getSettingsPages(),
                'services' => (new DivisionService())->getServices(true),
                'css_class' => implode(' ', [
                    StringHelper::getPrefixed('wrapper'),
                    StringHelper::getPrefixed('shortcode-generator'),
                ]),
                'parts' => $this->getParts(),
            ]);
        }

        public function makeRequest($url, $json_decode = true, $additional_headers = [])
        {
            $curl = curl_init();
            curl_setopt_array($curl, array_merge([
                CURLOPT_RETURNTRANSFER => 1,
                CURLOPT_CONNECTTIMEOUT => 0, // timeout
                CURLOPT_TIMEOUT => 1, // timeout
                CURLOPT_URL => $url,
                CURLOPT_HTTPHEADER => [
                    'User-Agent: keycdn-tools:https://jims.net',
                ],
            ], $additional_headers));
            $output = curl_exec($curl);
            curl_close($curl);

            return $json_decode ? json_decode($output, true) : $output;
        }
    }
}
