<?php

namespace JimsCustomerBooking\Helpers;

class ColorHelper extends Helper
{
    public static function foreground($bg_r, $bg_g = null, $bg_b = null)
    {
        $is_string = is_string($bg_r);
        if ($is_string) {
            list($bg_r, $bg_g, $bg_b) = self::hexToRgb($bg_r);
        }

        $contrast = self::contrast($bg_r, $bg_g, $bg_b, 255, 255, 255);

        if ($contrast >= 4.5) {
            $result = [255, 255, 255];
        } else {
            $result = [0, 0, 0];
        }

        if ($is_string) {
            $result = array_map('dechex', $result);
            $result = array_map(function ($v) {
                return strlen($v) == 1 ? "{$v}{$v}" : $v;
            }, $result);

            return '#' . implode('', $result);
        }

        return $result;
    }

    private static function contrast($r1, $g1, $b1, $r2, $g2, $b2)
    {
        $lum1 = self::luminance($r1, $g1, $b1);
        $lum2 = self::luminance($r2, $g2, $b2);
        $ratio = ($lum1 + 0.05) / ($lum2 + 0.05);

        if ($lum2 > $lum1) {
            $ratio = 1 / $ratio;
        }

        return $ratio;
    }

    private static function luminance($r, $g, $b)
    {
        $rgb = [$r, $g, $b];
        foreach ($rgb as $key => $val) {
            $val = $val / 255;
            $val = $val < 0.03928 ? $val / 12.92 : pow(($val + 0.055) / 1.055, 2.4);
            $rgb[$key] = $val;
        }

        return 0.2126 * $rgb[0] + 0.7152 * $rgb[1] + 0.0722 * $rgb[2];
    }

    /**
     * @param string $hex
     */
    private static function hexToRgb($hex)
    {
        $hex = str_replace('#', '', $hex);

        if (strlen($hex) == 3) {
            $r = hexdec(substr($hex, 0, 1) . substr($hex, 0, 1));
            $g = hexdec(substr($hex, 1, 1) . substr($hex, 1, 1));
            $b = hexdec(substr($hex, 2, 1) . substr($hex, 2, 1));
        } else {
            $r = hexdec(substr($hex, 0, 2));
            $g = hexdec(substr($hex, 2, 2));
            $b = hexdec(substr($hex, 4, 2));
        }

        return [$r, $g, $b];
    }
}
