<?php

namespace JimsCustomerBooking\Integrations;

/**
 * This class is the base class for all notifications.
 */
abstract class ANotification
{
    public const NOTIFICATION_SERVICE_EMAIL = 'email';

    public const NOTIFICATION_SERVICE_DIRECT = 'direct';

    public const NOTIFICATION_TYPE_NOTIFICATION = 'notification';

    public const NOTIFICATION_TYPE_ERROR = 'error';

    public const NOTIFICATION_TYPE_INFO = 'success';

    /**
     * @var bool
     */
    protected $test_mode = false;

    protected string $apiKey = '';

    protected string $service = '';

    protected mixed $service_provider = null;

    protected array $messages = [];

    protected array $service_response = [];

    /**
     * @param mixed $type
     */
    public function getMessage($type): array|string
    {
        return $this->messages[$type] ?? '';
    }

    /**
     * @param string $message
     * @param mixed  $type
     */
    public function setMessage(array|string $message, $type = self::NOTIFICATION_TYPE_NOTIFICATION): void
    {
        $this->messages[$type] = $message;
    }

    /**
     * @return bool
     */
    abstract public function send(string $type = self::NOTIFICATION_TYPE_NOTIFICATION);

    public function getMessages(): array
    {
        return $this->messages;
    }

    public function getServiceResponse(): array
    {
        return $this->service_response;
    }

    public function setServiceResponse(array $service_response): void
    {
        $this->service_response = $service_response;
    }

    abstract public function setupNotificationMessages(array $vars = []): void;

    /**
     * @param array $vars
     * @param mixed $message
     *
     * @return string|string[]
     */
    protected function replaceVars($message, $vars = []): array|string
    {
        // replace vars in messages
        // for each var in vars, replace the key with the value in the message
        foreach ($vars as $key => $value) {
            if (isset($vars[$key])) {
                $message = str_replace('{{' . $key . '}}', $value, $message);
            }
        }

        return $message;
    }
}
