<?php

namespace JimsCustomerBooking\Integrations;

use JimsCustomerBooking\Helpers\Http;
use JimsCustomerBooking\Helpers\OptionHelper;
use JimsGroup\Utilities\Logger;

class JimsResourcesIntegration
{
    protected $domain = 'https://resources.jims.net';

    protected $api_path = '/wp-json';

    protected $endpoints = [
        'get_subscribers' => '/newsletter/v2/subscribers',
        'subscribe' => '/newsletter/v2/subscribers',
    ];

    private $creds = [
        'client_key' => '961275ceb959038f70851454be9eb86036ecc11b',
        'client_secret' => '603e10ccf3098dec5f21ee10b04741934b6c9b10',
    ];

    public function isSubscribed($email, $list = 'WPCB'): bool
    {
        $subscribers = $this->getSubscribers();

        if (empty($subscribers)) {
            return false;
        }

        foreach ($subscribers as $subscriber) {
            if ($subscriber['email'] === $email) {
                return $this->isSubscriberInList($subscriber, $list);
            }
        }

        return false;
    }

    public function isSubscriberInList($subscriber, $list = 'WPCB')
    {
        if (isset($subscriber['lists']) && is_array($subscriber['lists'])) {
            return count(array_filter($subscriber['lists'], function ($item) use ($list) {
                return $item['name'] === $list;
            })) > 0;
        }

        return false;
    }

    /**
     * @param string $list
     *
     * @return mixed
     */
    public function getSubscribers($list = 'WPCB')
    {
        $url = $this->getEndpoint('get_subscribers');

        $response = Http::get($url, array_merge([
            'page' => 1,
            'per_page' => 1000,
        ], $this->creds), OptionHelper::isHTTPCompatibilityEnabled('Internal'));

        return empty($response['error']) ? json_decode($response['result'], true) : null;
    }

    /**
     * @param mixed $email
     * @param mixed $first_name
     * @param mixed $list
     *
     * @throws \Exception
     */
    public function subscribe($email, $first_name = '', $list = 'WPCB')
    {
        try {
            $url = $this->getEndpoint('subscribe');

            // add credentials to the url
            $url .= '?' . http_build_query($this->creds);

            $data = [
                'email' => $email,
                'first_name' => $first_name,
                'last_name' => null,
            ];

            // first name contains space split it to first name and last name
            if (str_contains($first_name, ' ')) {
                // split the name in two parts using the first space
                $name_parts = explode(' ', $first_name, 2);

                $data['first_name'] = $name_parts[0];
                $data['last_name'] = $name_parts[1];
            }

            $response = Http::post($url, $this->formatSubscriptionArray($data), [], OptionHelper::isHTTPCompatibilityEnabled('Internal'));

            return empty($response['error']) ? json_decode($response['result'], true) : null;
        } catch (\Exception $e) {
            Logger::error($e->getMessage());

            return null;
        }
    }

    public function formatSubscriptionArray($data)
    {
        return [
            'email' => $data['email'],
            'first_name' => $data['first_name'] ?? null,
            'last_name' => $data['last_name'] ?? null,
            'lists' => [
                [
                    'id' => 1,
                    'value' => 1,
                ],
            ],
            'list_N' => 0,
            'status' => 'confirmed',
        ];
    }

    protected function getEndpoint($name)
    {
        if (!isset($this->endpoints[$name])) {
            throw new \Exception('Endpoint not found');
        }

        return $this->domain . $this->api_path . $this->endpoints[$name];
    }
}
