<?php

namespace JimsCustomerBooking;

use JimsCustomerBooking\Helpers\StringHelper;

if (!class_exists('JimsCustomerBooking\JimsOptions')) {
    class JimsOptions
    {
        protected array $options = [
            'http_schema' => 'https',
            'domain' => 'cb.jims.net',
            'DivisionId' => null,
            'bookingFormDivisionId' => null,
            'countryCode' => 'AU',
            'jimsProperty' => null,
            'primaryColor' => null,
            'textColor' => null,
            'logoUrl' => null,
            'gtmId' => null,
            'gatId' => 'Analytics ID',
            'cssSelector' => null,
            'redirectOnly' => '1',
        ];

        public function __construct($options = null)
        {
            if (is_array($options) && $this->hasParams($options)) {
                $this->options = $options;
            }
        }

        public function __call(string $name, array $arguments)
        {
            return null;
        }

        public function __set(string $name, mixed $value): void
        {
            $this->options[$name] = $value;
        }

        public function __get(string $name): mixed
        {
            return $this->options[$name] ?? null;
        }

        public function addProperty($name, $value)
        {
            $this->options[$name] = $value;
        }

        /**
         * @param mixed $filtered
         *
         * @return array
         */
        public function getProperties($filtered = true)
        {
            $options = $this->options;

            if ($filtered) {
                $options = $this->filterOptions($options);
            }

            return apply_filters(StringHelper::getPrefixed('customer_booking_options'), $options);
        }

        protected function hasParams($option_arr): bool
        {
            $diff = array_diff(array_keys($this->options, $option_arr));

            return count($diff) == 0;
        }

        /**
         * Filters the options array
         * This fixes the bug with special characters in the cb url.
         *
         * @param mixed $options
         *
         * @return mixed
         */
        protected function filterOptions($options)
        {
            // filter # from hex color
            $options['primaryColor'] = str_replace('#', '', $options['primaryColor'] ?? '');
            $options['textColor'] = str_replace('#', '', $options['textColor'] ?? '');

            return $options;
        }
    }
}
