<?php

namespace JimsCustomerBooking;

use JimsCustomerBooking\Admin\BackendPages\CustomerBookingPage;
use JimsCustomerBooking\Admin\BackendPages\EmailSubscriptionSettingsPage;
use JimsCustomerBooking\Admin\BackendPages\FormCustomizeSettingsPage;
use JimsCustomerBooking\Admin\BackendPages\NativeFormSettingsPage;
use JimsCustomerBooking\Admin\BackendPages\SettingsPage;
use JimsCustomerBooking\Admin\BackendPages\ShortCodeGeneratorPage;
use JimsCustomerBooking\Database\Database;
use JimsCustomerBooking\Helpers\ColorHelper;
use JimsCustomerBooking\Helpers\OptionHelper;
use JimsCustomerBooking\Helpers\StringHelper;
use JimsCustomerBooking\Integrations\JimsLeadsIntegration;
use JimsCustomerBooking\RestfulAPI\AuthenticationController;
use JimsCustomerBooking\RestfulAPI\BookingsStatisticsController;
use JimsCustomerBooking\RestfulAPI\CustomerBookingController;
use JimsCustomerBooking\RestfulAPI\CustomerBookingCustomizeController;
use JimsCustomerBooking\RestfulAPI\EmailSubscriptionController;
use JimsCustomerBooking\RestfulAPI\InputValidationController;
use JimsCustomerBooking\RestfulAPI\OptionController;
use JimsCustomerBooking\RestfulAPI\ScheduleController;
use JimsCustomerBooking\RestfulAPI\ServiceController;
use JimsCustomerBooking\Shortcodes\BookingFormShortCode;
use JimsGroup\Utilities\Logger;
use JimsCustomerBooking\Helpers\FSHelper;

if (!class_exists('JimsCustomerBooking\JimsPlugin')) {
    class JimsPlugin
    {
        /**
         * --- PLEASE DO NOT CHANGE THIS ---
         * encryption key for the plugin
         * it should be a hexadecimal key, 136 characters long.
         */
        // phpcs:ignore
        public const ENCRYPTION_KEY = 'def000008eb640d6634a6d422b06ecbbd8ed8bd0fc417c4831b7aa7bfb7c89a4caa3973a13288795670bbc0793fffa622cb5c99fa407069efb64a11a1b1d40716f3cd6c0';

        /**
         * key for the option in wordpress options table.
         */
        public const OPTION_NAME = 'wpcb';
        public const SECKEY = 'jims-wpcb-communication-key-v1';

        /**
         * --- PLEASE DO NOT CHANGE THIS ---
         * menu slug for admin.
         */
        public const PREFIX = 'jims_wpcb';

        public const FMS_API_KEY = '3e8bce9e-be45-4499-b31a-23b6fb067aa7';

        public const FMS_TOKENS = [
            'secret' => 'yw86Dvcwwsm0vEpZ3oNRFrXkCNovNLKh',
            'fso_token' => 'Zm1zX3VzZXI6Q2I2MkFZaFRwOFdtYUJSMnBzUmpSNUxTTnNhRWY1cEw=',
            'fse_token' => 'amltc19qb2JzX21vYmlsZTpDNHFkOFl0OGVnWVVLMnRaZnlhNWtaUjVZR3JmNng0eQ==',
        ];

        public const FINDAJIM_TOKEN = '84bcb06b2d534989cf811f250b26dc10';

        // local is for testing and available only for local environments
        public const FMS_API_MODES = [
            'production', 'uat', 'hf', 'staging', 'int', 'development', 'local',
        ];
        public const VERSION = '4.0.5';

        private array $rest_classes = [];
        private array $registered = [];
        private ?JimsLeadsIntegration $leads = null;

        public function run(): void
        {
            add_action('plugins_loaded', [$this, 'activate']);
            add_action('init', [$this, 'setupShortcodes']);
            add_action('wp_enqueue_scripts', [$this, 'frontScripts']);
            add_action('admin_enqueue_scripts', [$this, 'adminScripts']);
            add_action('rest_api_init', [$this, 'registerRestAPI']);
            add_action(StringHelper::getPrefixed('jims_logo'), [$this, 'setupJimsLogo']);
            add_action('admin_notices', [$this, 'showDashboardNotifications']);
            add_action('init', [$this, 'registerBlocks']);
            add_action('wp_head', function () {
                $debug = boolval(OptionHelper::getFormOption('debugMode', 0));

                list($main_color) = $this->getFormColors();
                $foreground = ColorHelper::foreground($main_color);
                $themedColor = $foreground === '#000000' ? '#000000' : $main_color;
                $jsArgs = [
                    'wpjson' => get_rest_url(),
                ];

                if ($debug === true) {
                    $jsArgs['debug'] = boolval(OptionHelper::getFormOption('debugMode', 0));
                }

                // TODO: Clean this up and load a seperate host stylesheet
                ?><style>
                    :root {
                        --wpcb-color: <?= $main_color; ?>;
                        --wpcb-color-foreground: <?= $foreground; ?>;
                        --wpcb-color-themed: <?= $themedColor; ?>
                    }
                </style>
                <script>
                    window.wpcb = <?= json_encode($jsArgs); ?>;
                </script>
            <?php
            });

            // modify shortcode block's callback
            add_filter('register_block_type_args', [$this, 'modifyShortcodeBlock'], 1, 2);

            // Checks the theme and theme version and modifies settings to match
            $this->setupCompatibility();
            $this->registerBackendPages();
            $this->leads = JimsLeadsIntegration::getInstance();
            $this->setupMiddleWares();
        }

        public function registerBlocks(): void
        {
            // Pull all the block json files
            $blocks = FSHelper::glob('build/blocks/*/block.json');

            foreach ($blocks as $path) {
                // register_block_type(substr($path, 0, -11));
                register_block_type($path);
            }
        }

        public function frontScripts(): void
        {
            $frontAsset = FSHelper::require('build/front.asset.php');

            wp_register_script(
                StringHelper::getPrefixed('js'),
                FSHelper::url('build/front.js'),
                $frontAsset['dependencies'],
                $frontAsset['version'],
                true
            );

            wp_enqueue_script(StringHelper::getPrefixed('js'));

            $hostAsset = FSHelper::require('build/host.asset.php');
            wp_enqueue_style(
                StringHelper::getPrefixed('js'),
                FSHelper::url('build/host.css'),
                [],
                $hostAsset['version'],
            );
        }

        public function adminScripts(): void
        {
            $htmlAsset = FSHelper::require('build/html-extension.asset.php');

            wp_enqueue_script(
                StringHelper::getPrefixed('html-extension'),
                FSHelper::url('build/html-extension.js'),
                $htmlAsset['dependencies'],
                $htmlAsset['version'],
            );

            $adminAsset = FSHelper::require('build/admin.asset.php');

            wp_register_script(
                StringHelper::getPrefixed('admin-js'),
                FSHelper::url('build/admin.js'),
                $adminAsset['dependencies'],
                $adminAsset['version'],
                true
            );

            OptionHelper::localizeFormVariables('admin-js');

            wp_enqueue_script(StringHelper::getPrefixed('admin-js'));

            wp_enqueue_style(
                StringHelper::getPrefixed('admin-style'),
                FSHelper::url('build/admin.css'),
                [],
                $adminAsset['version'],
            );

            // enqueue fontawesome if not theme is not the new jims theme
            $themeName = strtolower(wp_get_theme()->get('Name'));

            if (!str_contains($themeName, 'jims') || str_contains($themeName, 'enfold')) {
                wp_register_style(
                    StringHelper::getPrefixed('fontawesome-solid'),
                    FSHelper::url('build/fonts/css/solid.css'),
                    [],
                    $adminAsset['version'],
                );

                wp_enqueue_style(
                    StringHelper::getPrefixed('fontawesome-style'),
                    FSHelper::url('build/fonts/css/fontawesome.css'),
                    [
                        StringHelper::getPrefixed('fontawesome-solid'),
                    ],
                    $adminAsset['version'],
                );
            }
        }

        private function registerBackendPages(): array
        {
            return $this->registered['backend_pages'] = [
                SettingsPage::class => new SettingsPage(),
                NativeFormSettingsPage::class => new NativeFormSettingsPage(),
                FormCustomizeSettingsPage::class => new FormCustomizeSettingsPage(),
                EmailSubscriptionSettingsPage::class => new EmailSubscriptionSettingsPage(),
                CustomerBookingPage::class => new CustomerBookingPage(),
                ShortCodeGeneratorPage::class => new ShortCodeGeneratorPage(),
            ];
        }

        public function showDashboardNotifications(): void
        {
            if (!$this->isUserSubscribed()) {
                printf(
                    '<div class="notice notice-error is-dismissible"><p>%s <a href="%s">here</a></p></div>',
                    'WP CB form is <strong>LOCKED</strong>! To unlock please subscribe to our email service ',
                    admin_url('admin.php?page=wpcb_subscription_settings.php')
                );
            }

            if ($this->leads->getSiteStatus() === false) {
                printf(
                    '<div class="notice notice-error is-dismissible"><p>%s <a href="%s">%s</a> %s</p></div>',
                    // phpcs:ignore
                    'Your website appears to be actively blocking Jim\'s Group monitoring servers. This could potentially affect the health and error tracking of your CB form. Please ',
                    'mailto:infotech@jims.net',
                    'contact support',
                    'for more information.'
                );
            }
        }

        private function isUserSubscribed(): bool
        {
            $options = OptionHelper::get(EmailSubscriptionSettingsPage::OPTION_NAME, []);

            return isset($options['email'])
                && strlen($options['email']) > 3
                && str_contains($options['email'], '@');
        }

        public function setupShortcodes(): void
        {
            if ($this->isUserSubscribed()) {
                $this->registered['shortcodes'] = [
                    BookingFormShortCode::class => new BookingFormShortCode(),
                ];
            }
        }

        public function registerRestAPI(): void
        {
            $this->rest_classes = [
                InputValidationController::class => new InputValidationController(),
                OptionController::class => new OptionController(),
                ServiceController::class => new ServiceController(),
                AuthenticationController::class => new AuthenticationController(),
                CustomerBookingController::class => new CustomerBookingController(),
                BookingsStatisticsController::class => new BookingsStatisticsController(),
                CustomerBookingCustomizeController::class => new CustomerBookingCustomizeController(),
                EmailSubscriptionController::class => new EmailSubscriptionController(),
                ScheduleController::class => new ScheduleController(),
            ];

            // register rest api controllers hook
            $this->rest_classes = apply_filters(self::PREFIX . '_rest_controller_register', $this->rest_classes);
        }

        public function modifyShortcodeBlock($settings, $name): array
        {
            if ($name === 'core/shortcode') {
                $settings['render_callback'] = function ($attributes, $content) {
                    if(str_contains($content, 'wpcb')){
                        return $content;
                    }

                    return render_block_core_shortcode($attributes, $content);
                };
            }

            return $settings;
        }

        public static function getOptions(): JimsOptions
        {
            return OptionHelper::getEmbeddedFormOptions();
        }

        public function activate(): void
        {
            $option_name = StringHelper::getPrefixed('version', 'machine');
            $current_version = get_option($option_name, '0.0.0');

            if ($current_version != JIMS_CBP_VERSION) {
                update_option($option_name, JIMS_CBP_VERSION, true);
                Database::setupOrUpdateDatabase($current_version);

                // update website on leads.jims.net
                $res = $this->updateWebsiteToLeads();
                Logger::info(is_wp_error($res) ? $res->get_error_message() : $res);
            }
        }

        private function setupCompatibility(): void
        {
            // check if enfold theme is active and get logo from theme options
            add_filter(StringHelper::getPrefixed('booking_form_logo_url'), function ($logo) {
                $theme = wp_get_theme();
                $theme_name = $theme->get('Name');

                if (str_contains(strtolower($theme_name), 'enfold')) {
                    $options = get_option('avia_options_jims_enfold');
                    $logo = isset($options['avia'], $options['avia']['logo']) ? $options['avia']['logo'] : '';
                }

                return $logo;
            }, 99);
        }

        public function getRegistered(string $name): array
        {
            return isset($this->registered[$name])
                ? $this->registered[$name]
                : $this->registered;
        }

        public function setupJimsLogo(): void
        {
            printf(
                '<img src="%s" alt="%s">',
                FSHelper::url('build/img/graphics/jims.png'),
                'Jims Logo'
            );
        }

        private function getFormColors(): array
        {
            $main_color = OptionHelper::getValue(NativeFormSettingsPage::OPTION_NAME, 'bookingFormMainColor', '');
            if (empty($main_color)) {
                $main_color = '#0b75b4';
            }

            return [$main_color];
        }

        private function updateWebsiteToLeads(): array|\WP_Error
        {
            return $this->leads->updateWebsite();
        }

        private function setupMiddleWares(): array
        {
            return $this->registered['middlewares'] = [
                // Http::class => new Http()
            ];
        }
    }
}
