<?php

namespace JimsCustomerBooking\RestfulAPI;

use JimsCustomerBooking\JimsPlugin;
use JimsCustomerBooking\Services\TokenManager;

class AuthenticationController extends RestController
{
    protected string $route = 'auth';

    public function registerRoutes(): void
    {
        register_rest_route($this->getNamespace(), '/' . $this->getRoute() . '/get', [
            [
                'methods' => \WP_REST_Server::CREATABLE,
                'callback' => [$this, 'generateToken'],
                'permission_callback' => [$this, 'checkPermissions'],
            ],
        ]);
    }

    public function generateToken(\WP_REST_Request $request)
    {
        // random string
        $random_string = TokenManager::generateRandomString(random_int(10, 20));

        // set header for no cache
        header('Cache-Control: no-cache, must-revalidate');

        if ($request->has_param('duration')) {
            $duration = $request->get_param('duration');
        } else {
            $duration = '10 minutes';
        }

        return $this->response([
            'token' => TokenManager::encode($this->key, JimsPlugin::PREFIX . $random_string),
        ]);
    }
}
