<?php

/**
 * Created by Konstantinos Tsatsarounos.
 * Date: 4/8/2019
 * Time: 10:55 AM.
 */

namespace JimsCustomerBooking\RestfulAPI;

use JimsCustomerBooking\Helpers\OptionHelper;
use JimsCustomerBooking\Helpers\StringHelper;

if (!class_exists('JimsCustomerBooking\RestfulAPI\CustomerBookingCustomizeController')) {
    class CustomerBookingCustomizeController extends RestController
    {
        protected string $route = 'customize';

        public function registerRoutes(): void
        {
            register_rest_route(
                $this->getNamespace(),
                $this->getRoute() . '/service-order',
                [
                    [
                        'methods' => \WP_REST_Server::CREATABLE,
                        'callback' => [$this, 'updateServiceOrder'],
                        'permission_callback' => '__return_true',
                    ],
                    [
                        'methods' => \WP_REST_Server::READABLE,
                        'callback' => [$this, 'getServiceOrder'],
                        'permission_callback' => '__return_true',
                    ],
                ]
            );

            register_rest_route(
                $this->getNamespace(),
                $this->getRoute() . '/service-groups',
                [
                    [
                        'methods' => \WP_REST_Server::CREATABLE,
                        'callback' => [$this, 'updateServiceGroups'],
                        'permission_callback' => '__return_true',
                    ],
                    [
                        'methods' => \WP_REST_Server::READABLE,
                        'callback' => [$this, 'getServiceGroups'],
                        'permission_callback' => '__return_true',
                    ],
                ]
            );

            register_rest_route(
                $this->getNamespace(),
                $this->getRoute() . '/service-list',
                [
                    [
                        'methods' => \WP_REST_Server::CREATABLE,
                        'callback' => [$this, 'updateServiceList'],
                        'permission_callback' => '__return_true',
                    ],
                    [
                        'methods' => \WP_REST_Server::READABLE,
                        'callback' => [$this, 'getServiceList'],
                        'permission_callback' => '__return_true',
                    ],
                ]
            );

            register_rest_route(
                $this->getNamespace(),
                $this->getRoute() . '/service-descriptions',
                [
                    [
                        'methods' => \WP_REST_Server::CREATABLE,
                        'callback' => [$this, 'updateServiceDescriptions'],
                        'permission_callback' => '__return_true',
                    ],
                    [
                        'methods' => \WP_REST_Server::READABLE,
                        'callback' => [$this, 'getServiceDescriptions'],
                        'permission_callback' => '__return_true',
                    ],
                ]
            );

            register_rest_route(
                $this->getNamespace(),
                $this->getRoute() . '/options',
                [
                    [
                        'methods' => \WP_REST_Server::CREATABLE,
                        'callback' => [$this, 'updateOptions'],
                        'permission_callback' => '__return_true',
                    ],
                    [
                        'methods' => \WP_REST_Server::READABLE,
                        'callback' => [$this, 'getOptions'],
                        'permission_callback' => '__return_true',
                    ],
                ]
            );

            register_rest_route(
                $this->getNamespace(),
                $this->getRoute() . '/delete-all-options',
                [
                    [
                        'methods' => \WP_REST_Server::CREATABLE,
                        'callback' => [$this, 'deleteAllOptions'],
                        'permission_callback' => '__return_true',
                    ],
                ]
            );
        }

        /**
         * It saves the service order as an array of objects in the database
         * every object has the service id and the gravity number.
         *
         * The gravity number defines the order of the service in the booking form
         *
         * @return \WP_REST_Response
         */
        public function updateServiceOrder(\WP_REST_Request $request)
        {
            list($result, $data) = $this->saveOption($request, 'customer_booking_service_order');

            return $this->response([
                'result' => $result ? 'success' : 'failure',
                'data' => isset($data) ? $data : [],
            ], 200);
        }

        /**
         * @return \WP_REST_Response
         */
        public function getServiceOrder(\WP_REST_Request $request)
        {
            $option_name = $this->getOptionName('customer_booking_service_order');

            $data = get_option($option_name, []);

            return $this->response([
                'result' => !empty($data) ? 'success' : 'failure',
                'data' => $data,
            ], 200);
        }

        /**
         * @return \WP_REST_Response
         */
        public function updateServiceGroups(\WP_REST_Request $request)
        {
            list($result, $data) = $this->saveOption($request, 'customer_booking_service_groups');

            return $this->response([
                'result' => $result ? 'success' : 'failure',
                'data' => isset($data) ? $data : [],
            ], 200);
        }

        /**
         * Returns the service ids in a grouped manner, with descriptions.
         *
         * @return \WP_REST_Response
         */
        public function getServiceGroups(\WP_REST_Request $request)
        {
            $option_name = $this->getOptionName('customer_booking_service_groups');
            $data = get_option($option_name, []);

            return $this->response([
                'result' => !empty($data) ? 'success' : 'failure',
                'data' => $data,
            ], 200);
        }

        /**
         * @return \WP_REST_Response
         */
        public function updateServiceList(\WP_REST_Request $request)
        {
            error_log(json_encode($request->get_body()), 0);

            return $this->response(['data' => $request->get_body()], 200);
        }

        /**
         * Service List is a list of preselected services, it's like a package.
         *
         * @return \WP_REST_Response
         */
        public function getServiceList(\WP_REST_Request $request)
        {
            error_log(json_encode($request->get_body()), 0);

            return $this->response(['data' => $request->get_body()], 200);
        }

        public function updateServiceDescriptions(\WP_REST_Request $request)
        {
            list($result, $data) = $this->saveOption($request, 'customer_booking_service_descriptions');

            return $this->response([
                'result' => $result ? 'success' : 'failure',
                'data' => isset($data) ? $data : [],
            ], 200);
        }

        public function getServiceDescriptions(\WP_REST_Request $request)
        {
            $option_name = $this->getOptionName('customer_booking_service_descriptions');
            $data = get_option($option_name, []);

            return $this->response([
                'result' => !empty($data) ? 'success' : 'failure',
                'data' => $data,
            ], 200);
        }

        public function updateOptions(\WP_REST_Request $request)
        {
            list($result, $data) = $this->saveOption($request, 'customer_booking_customizer_options');

            return $this->response([
                'result' => $result ? 'success' : 'failure',
                'data' => isset($data) ? $data : [],
            ], 200);
        }

        public function getOptions(\WP_REST_Request $request)
        {
            $option_name = $this->getOptionName('customer_booking_customizer_options');
            $data = get_option($option_name, []);

            return $this->response([
                'result' => !empty($data) ? 'success' : 'failure',
                'data' => $data,
            ], 200);
        }

        public function saveOption(\WP_REST_Request $request, string $optionName): array
        {
            $data = [];
            $body = $request->get_body();
            $option_name = $this->getOptionName($optionName);

            $result = false;

            if (StringHelper::isJson($body)) {
                $data = json_decode($body, true);

                if (!empty($data)) {
                    $result = update_option($option_name, $data);
                }
            }

            return [$result, $data];
        }

        public function deleteAllOptions(\WP_REST_Request $request)
        {
            $option_name = $this->getOptionName('customer_booking_service_order');
            delete_option($option_name);

            $option_name = $this->getOptionName('customer_booking_service_groups');
            delete_option($option_name);

            $option_name = $this->getOptionName('customer_booking_service_descriptions');
            delete_option($option_name);

            $option_name = $this->getOptionName('customer_booking_customizer_options');
            delete_option($option_name);

            $this->deleleDepreciatedOptions();

            return $this->response([
                'result' => 'success',
                'data' => [],
            ], 200);
        }

        public function deleleDepreciatedOptions(): void
        {
            delete_option(StringHelper::getPrefixed('customer_booking_service_order', 'machine'));
            delete_option(StringHelper::getPrefixed('customer_booking_service_groups', 'machine'));
            delete_option(StringHelper::getPrefixed('customer_booking_service_descriptions', 'machine'));
        }

        public function getOptionKey(string $key): string
        {
            $division_id = OptionHelper::getFormOption('bookingFormDivisionId');

            return empty($division_id)
                ? $key
                : 'division_' . $division_id . '_' . $key;
        }

        /**
         * @param mixed $name
         */
        public function getOptionName($name): string
        {
            $key = $this->getOptionKey($name);

            return StringHelper::getPrefixed($key, 'machine');
        }
    }
}
