<?php

namespace JimsCustomerBooking\RestfulAPI;

use JimsCustomerBooking\Admin\BackendPages\EmailSubscriptionSettingsPage;
use JimsCustomerBooking\Helpers\OptionHelper;
use JimsCustomerBooking\Integrations\JimsResourcesIntegration;

class EmailSubscriptionController extends RestController
{
    protected string $route = 'email-subscription';

    public function registerRoutes(): void
    {
        register_rest_route($this->getNamespace(), '/' . $this->getRoute() . '/create', [
            [
                'methods' => \WP_REST_Server::CREATABLE,
                'callback' => [$this, 'subscribe'],
                'args' => [
                    'email' => [
                        'required' => true,
                        'validate_callback' => function ($param, $request, $key) {
                            return is_email($param);
                        },
                    ],
                ],
                'permission_callback' => '__return_true',
            ],
        ]);

        register_rest_route($this->getNamespace(), '/' . $this->getRoute() . '/subscribed', [
            [
                'methods' => \WP_REST_Server::READABLE,
                'callback' => [$this, 'isSubscribed'],
                'args' => [
                    'email' => [
                        'required' => true,
                        'validate_callback' => function ($param, $request, $key) {
                            return is_email($param);
                        },
                    ],
                ],
                'permission_callback' => '__return_true',
            ],
        ]);

        register_rest_route($this->getNamespace(), '/' . $this->getRoute() . '/get', [
            [
                'methods' => \WP_REST_Server::READABLE,
                'callback' => [$this, 'getSubscription'],
                'permission_callback' => '__return_true',
            ],
        ]);
    }

    public function getSubscription(\WP_REST_Request $request): \WP_REST_Response
    {
        $options = OptionHelper::get(EmailSubscriptionSettingsPage::OPTION_NAME, []);

        return $this->response([
            'email' => $options['email'] ?? '',
            'name' => $options['name'] ?? '',
        ]);
    }

    /**
     * @throws \Exception
     */
    public function subscribe(\WP_REST_Request $request): \WP_REST_Response
    {
        $email = $request->get_param('email');
        $name = $request->get_param('name');

        $saved = OptionHelper::update(EmailSubscriptionSettingsPage::OPTION_NAME, [
            'email' => $email,
            'name' => $name,
        ]);

        $integration = new JimsResourcesIntegration();
        $response = $integration->subscribe($email, $request->get_param('name'));

        return !empty($response) ? $this->response([
            'status' => true,
            'saved' => $saved,
            'message' => 'Email subscribed successfully',
        ]) : $this->response([
            'status' => false,
            'saved' => $saved,
            'message' => 'Failed to subscribe email',
        ], 400);
    }

    public function isSubscribed(\WP_REST_Request $request): \WP_REST_Response
    {
        $integration = new JimsResourcesIntegration();

        return $this->response([
            'status' => $integration->isSubscribed($request->get_param('email')),
        ]);
    }
}
