<?php

namespace JimsCustomerBooking\RestfulAPI;

use libphonenumber\geocoding\PhoneNumberOfflineGeocoder as libPhoneNumberOfflineGeocoder;
use libphonenumber\NumberParseException;
use libphonenumber\PhoneNumberUtil as libPhoneNumberUtil;

class InputValidationController extends RestController
{
    protected string $route = 'input-validation';

    public function registerRoutes(): void
    {
        register_rest_route($this->getNamespace(), '/' . $this->route . '/phone', [
            [
                'methods' => \WP_REST_Server::READABLE,
                'callback' => [$this, 'validatePhone'],
                'permission_callback' => '__return_true',
            ],
        ]);
    }

    /**
     * @return \WP_REST_Response
     */
    public function validatePhone(\WP_REST_Request $request)
    {
        $phone = $request->get_param('number');
        $country = $request->get_param('country');
        $isValid = false;
        $data = [
            'phone' => $phone,
            'country' => $country,
        ];

        try {
            $phoneUtil = libPhoneNumberUtil::getInstance();
            $phoneNumber = $phoneUtil->parse($phone, $country);
            $geocoder = libPhoneNumberOfflineGeocoder::getInstance();

            $isValid = $phoneUtil->isValidNumber($phoneNumber);
            $data['description'] = $geocoder->getDescriptionForNumber($phoneNumber, 'en_US');
        } catch (NumberParseException $e) {
            error_log($e->getMessage());
        }

        return rest_ensure_response([
            'result' => $isValid,
            'data' => $data,
        ]);
    }
}
