<?php

namespace JimsCustomerBooking\RestfulAPI;

use JimsCustomerBooking\Admin\BackendPages\NativeFormSettingsPage;
use JimsCustomerBooking\Database\Database;
use JimsCustomerBooking\Helpers\OptionHelper;
use JimsCustomerBooking\Helpers\StringHelper;
use JimsCustomerBooking\Integrations\JimsLeadsIntegration;

if (!class_exists('JimsCustomerBooking\RestfulPI\OptionController')) {
    class OptionController extends RestController
    {
        protected string $route = 'options';

        public function registerRoutes(): void
        {
            register_rest_route($this->getNamespace(), $this->getRoute() . '/form-events', [
                [
                    'methods' => \WP_REST_Server::CREATABLE,
                    'callback' => [$this, 'updateEventsForm'],
                    'permission_callback' => '__return_true',
                ],
                [
                    'methods' => \WP_REST_Server::READABLE,
                    'callback' => [$this, 'getEventsForm'],
                    'permission_callback' => '__return_true',
                ],
            ]);

            register_rest_route($this->getNamespace(), $this->getRoute() . '/clear-transients', [
                [
                    'methods' => \WP_REST_Server::CREATABLE,
                    'callback' => [$this, 'clearTransients'],
                    'permission_callback' => '__return_true',
                ],
            ]);

            register_rest_route($this->getNamespace(), $this->getRoute() . '/leads', [
                [
                    'methods' => \WP_REST_Server::CREATABLE,
                    'callback' => [$this, 'updateLeadsWebsite'],
                    'permission_callback' => '__return_true',
                ],
            ]);

            register_rest_route($this->getNamespace(), $this->getRoute() . '/form-events-admin', [
                [
                    'methods' => \WP_REST_Server::CREATABLE,
                    'callback' => [$this, 'deleteEventsForm'],
                    'permission_callback' => '__return_true',
                ],
            ]);
        }

        /**
         * @return \WP_REST_Response
         */
        public function clearTransients(\WP_REST_Request $request)
        {
            $count = Database::deleteAllTransients();

            return $this->response([
                'message' => $count ? 'success' : 'failure',
                'count' => $count,
            ], $count ? 200 : 500);
        }

        /**
         * @return \WP_REST_Response
         */
        public function updateLeadsWebsite(\WP_REST_Request $request)
        {
            $integration = new JimsLeadsIntegration();

            $division_id = OptionHelper::getFormOption('bookingFormDivisionId');
            $country = OptionHelper::getFormOption('country');
            $debug_mode = OptionHelper::getFormOption('debugMode', 0);

            $integration->updateSite($division_id, $country, $debug_mode);

            return $this->response([
                'message' => 'success',
            ], 200);
        }

        /**
         * @param \WP_REST_Request $request
         *
         * @return \WP_REST_Response
         */
        public function updateEventsForm($request)
        {
            $options = $request->get_params();
            $result = update_option(
                StringHelper::getPrefixed(NativeFormSettingsPage::OPTION_NAME, 'machine'),
                $options
            );

            // update leads with website data
            $leads = JimsLeadsIntegration::getInstance();
            $leads->updateWebsite();

            return $this->response([
                'data' => $options,
                'status' => $result ? 200 : 500,
                'message' => $result ? 'success' : 'failure',
            ]);
        }

        public function deleteEventsForm($request)
        {
            $params = $request->get_params();

            $result = false;
            if (isset($params['allow']) && $params['allow'] == 'admin_' . md5('admin')) {
                $result = delete_option(
                    StringHelper::getPrefixed(NativeFormSettingsPage::OPTION_NAME, 'machine')
                );
            }

            return $this->response([
                'status' => $result ? 200 : 500,
                'message' => $result ? 'success' : 'failure',
            ]);
        }

        /**
         * @param \WP_REST_Request $request
         *
         * @return \WP_REST_Response
         */
        public function getEventsForm($request)
        {
            $form = OptionHelper::get(NativeFormSettingsPage::OPTION_NAME, []);

            return $this->response($form);
        }
    }
}
