<?php

namespace JimsCustomerBooking\RestfulAPI;

use JimsCustomerBooking\Helpers\HashHelper;
use JimsCustomerBooking\JimsPlugin;

class ScheduleController extends RestController
{
    protected string $route = 'schedule';

    public function registerRoutes(): void
    {
        $namespace = $this->getNamespace();

        register_rest_route($namespace, $route = '/' . $this->getRoute() . '/create', [
            [
                'methods' => \WP_REST_Server::CREATABLE,
                'callback' => [$this, 'setSchedule'],
                'permission_callback' => [$this, 'checkPermissions'],
            ],
        ]);
    }

    public function setSchedule($request): \WP_REST_Response
    {
        $text = $request->get_param('text');

        try {
            $encryptedText = HashHelper::deHash($text, JimsPlugin::ENCRYPTION_KEY);
        } catch (\Exception $e) {
            return new \WP_REST_Response([
                'error' => $e->getMessage(),
            ], 500);
        }

        return new \WP_REST_Response([
            'encrypted_text' => $encryptedText,
        ], 200);
    }
}
