<?php

namespace JimsCustomerBooking\Shortcodes;

use JimsCustomerBooking\Admin\BackendPages\NativeFormSettingsPage;
use JimsCustomerBooking\Helpers\OptionHelper;
use JimsCustomerBooking\Helpers\StringHelper;
use JimsCustomerBooking\Template;
use JimsCustomerBooking\Helpers\FSHelper;

class BookingFormShortCode extends Shortcode
{
    protected string $template = 'shortcode.booking_form';
    protected array $default_attributes = [];
    protected array $available_url_params = [
        'division_id' => 'divisionId',
        'service_list' => 'serviceListName',
        'country' => 'countryCode',
    ];

    /**
     * Order matters, don't change the order if you don't know what you are doing.
     */
    protected array $step_templates = [
        'divisions' => 'partial.booking_form-step_divisions',
        'services' => 'partial.booking_form-step_services',
        'address' => 'partial.booking_form-step_address',
        'client_info' => 'partial.booking_form-step_client_info',
    ];

    protected array $work_guarantee_templates = [
        'work_guarantee_button' => 'partial.guarantee-button',
        'work_guarantee_dialog' => 'partial.guarantee-dialog',
    ];

    public function configuration(): array
    {
        return [
            'shortcode_id' => StringHelper::getPrefixed('booking_form', 'machine'),
            'label' => 'Booking Form',
            'template' => $this->template,
        ];
    }

    public function handler($atts, $content)
    {
        $component_id = mt_rand(111, 9999);
        $division_id = $this->getFormOption('bookingFormDivisionId');
        $css_classes = ['wpcb'];
        $customize_options = $this->getCustomizeOption('customer_booking_customizer_options', [], $division_id);

        $_atts = $this->getAtts([
            'show_step_headings' => false,
            'hide_step_search' => false,
            'accept_url_params' => 0,
            'allowed_url_params' => '',
            'division_id' => (int) $division_id,
            'country' => $this->getFormOption('country'), // this is the default countryCode (AU, NZ, etc)
            'phone_number' => $this->getFormOption('phone'),
            'address_text' => $this->getFormOption('bookingFormAddressText'),
            'details_text' => $this->getFormOption('bookingFormDetailsText'),
            'description_text' => $this->getFormOption('bookingFormDescriptionText'),
            'confirm_address' => $this->getFormOption('confirmAddress', 0),
            'iframes_allowed' => $this->getFormOption('iframesAllowed', 0),
            'template' => $this->template,
            'service_ids' => [],
            'component_id' => $component_id,
            'logo_url' => apply_filters(StringHelper::getPrefixed('booking_form_logo_url'), $this->getLogoUrl()),
            'hide_logo' => 0,
            'contained' => 0,
            'show_country' => true, // enables or disables country select field
            'show_info_text' => true, // shows the form labels
            'class' => implode(' ', $css_classes),
            'prefix_class' => StringHelper::getPrefix(StringHelper::PREFIXED_TYPE_SLUG),
            'button_text' => 'Submit',
            'enable_popup' => 0,
            'show_steps' => 'services,client_info',
            'show_only_services' => [],
            'show_only_divisions' => [],
            'exclude_services' => [],
            'service_list' => '',
            'subscribe_text' => 'Subscribe for special offers from Jim\'s Group',
            'grouping_enabled' => isset($customize_options['categories_enabled'])
            && $customize_options['categories_enabled'] == true
                ? 1
                : 0,
            'custom_order_enabled' => isset($customize_options['custom_order'])
            && $customize_options['custom_order'] == true
                ? 1
                : 0,
            'alphabetical_order' => isset($customize_options['alphabetical_order'])
            && $customize_options['alphabetical_order'] == true
                ? 1
                : 0,
        ]);
        $ignore_grouping = false;

        if (isset($atts['class']) && !empty($atts['class'])) {
            $atts['class'] = implode(' ', $css_classes) . ' ' . $atts['class'];
        }

        if (isset($atts['service_ids']) && !empty($atts['service_ids'])) {
            $atts['service_ids'] = explode(',', $atts['service_ids']);
        }

        // todo refactor this
        $subscribe_text_country_override = false;
        $subscribe_text_country_override_key = '';
        $subscribe_text_country_override_value = '';
        foreach ($atts as $key => $value) {
            if (str_starts_with($key, 'subscribe_text') && preg_match('/^subscribe_text_[a-z]{2}$/', $key)) {
                $subscribe_text_country_override = true;
                $subscribe_text_country_override_key = $key;
                $subscribe_text_country_override_value = $value;
            }
        }

        // apply filters, modify attributes
        $atts = apply_filters(
            StringHelper::getPrefixed('booking_form_atts'),
            shortcode_atts($_atts, $atts, $this->shortcode_id),
            $content,
            $this->shortcode_id
        );

        // Force the division_id to be an integer as the javascript checks the
        // type of the value
        if(array_key_exists('division_id', $atts)){
            $atts['division_id'] = (int) $atts['division_id'];
        }

        $applicable_params = $this->getApplicableURLParams($atts['allowed_url_params']);

        if ($atts['accept_url_params'] == 1) {
            foreach ($applicable_params as $param => $url_param) {
                $atts[$param] = $this->getURLParam($url_param, $atts[$param]);
            }
        }

        // todo refactor this
        // check if there is an $att which starts with subscribe_text and matches
        // the pattern subscribe_text_{country_code}
        if (
            $subscribe_text_country_override
            && strtolower($atts['country']) === strtolower(substr($subscribe_text_country_override_key, -2))
        ) {
            $atts['subscribe_text'] = $subscribe_text_country_override_value;
        }

        if (!empty($atts['service_ids'])) {
            $atts['show_steps'] = 'client_info';
        }

        $available_steps = explode(',', $atts['show_steps']);


        if (!empty($atts['show_steps']) && str_contains($atts['show_steps'], 'divisions')) {
            $atts['division_id'] = null;
        }

        // remove divisions step if allowed url params is enabled and divisionId is set
        if (
            $atts['accept_url_params'] == 1
            && array_key_exists('division_id', $applicable_params)
            && !empty($_GET['divisionId'])
        ) {
            $atts['division_id'] = (int) $_GET['divisionId'];

            $ignore_grouping = true;
            $available_steps = array_filter($available_steps, function ($step) {
                return $step !== 'divisions';
            });
        }

        if ($atts['grouping_enabled'] === 1 && (count($available_steps) > 2 || $ignore_grouping)) {
            $atts['grouping_enabled'] = 0;
        }

        if (isset($atts['contained']) && $atts['contained'] == 1) {
            $atts['class'] .= ' is-contained';
        }

        if (isset($atts['enable_popup']) && $atts['enable_popup'] == 1) {
            $atts['class'] .= ' is-popup';
        }

        // show only selected services
        if (isset($atts['show_only_services']) && !empty($atts['show_only_services'])) {
            $atts['show_only_services'] = explode(',', $atts['show_only_services']);
        }

        // show only selected divisions
        if (isset($atts['show_only_divisions']) && !empty($atts['show_only_divisions'])) {
            $atts['show_only_divisions'] = explode(',', $atts['show_only_divisions']);
        }

        // exclude services
        if (isset($atts['exclude_services']) && !empty($atts['exclude_services'])) {
            $atts['exclude_services'] = explode(',', $atts['exclude_services']);
        }

        $atts['required_fields'] = OptionHelper::getRequiredInfoFields();

        // load steps
        $step_templates = $this->loadSteps($available_steps);

        $filter_names = [];
        if (count($step_templates) < count($available_steps)) {
            $steps_html = '';
            error_log('step_templates count is less than step_templates count');
        } else {
            $steps_html = $this->renderSteps($step_templates, $atts, $filter_names);
        }

        // add steps to attributes
        $atts['steps'] = $steps_html;
        $atts['steps_count'] = count($available_steps);

        // add work guarantee templates
        $work_guarantee_templates = $this->getWorkGuaranteeTemplates();
        foreach ($work_guarantee_templates as $name => $template) {
            $filter_name = StringHelper::getPrefixed(
                'booking_form_' . $name . '_html',
                'machine'
            );
            $atts[$name] = apply_filters($filter_name, $template->get($atts), $this->shortcode_id, $atts);
        }

        // get template object, it contains override if the shortcode has template attribute
        $template = new Template($atts['template']);

        $atts['show_step_headings'] = boolval($atts['show_step_headings']);
        $atts['hide_step_search'] = boolval($atts['hide_step_search']);


        // render template, allows overriding the html
        return apply_filters(
            StringHelper::getPrefixed('form_html'),
            $template->get($atts + ['content' => $content]),
            $atts,
            $content,
            $this->shortcode_id
        );
    }

    /**
     * @param mixed $allowed_url_params
     *
     * @return mixed|string[]
     */
    public function getApplicableURLParams($allowed_url_params): mixed
    {
        $allowed_url_params = explode(',', $allowed_url_params);

        return array_filter($this->available_url_params, function ($param, $key) use ($allowed_url_params) {
            return in_array($param, $allowed_url_params);
        }, ARRAY_FILTER_USE_BOTH);
    }

    /**
     * Checks a URL parameter and returns the value if it exists, otherwise returns the default value.
     *
     * @param mixed      $name
     * @param mixed      $default
     * @param null|mixed $validationCallback
     *
     * @return mixed|string
     */
    public function getURLParam($name, $default = '', $validationCallback = null)
    {
        $value = $_GET[$name] ?? $default;

        if ($validationCallback !== null) {
            return $validationCallback($value) ? $value : $default;
        }

        return $value;
    }

    /**
     * @param mixed $name
     * @param mixed $default
     *
     * @return mixed|string
     */
    public function getFormOption($name, $default = '')
    {
        $options = OptionHelper::get(NativeFormSettingsPage::OPTION_NAME, []);

        return $options[$name] ?? $default;
    }

    /**
     * @param mixed      $name
     * @param mixed      $default
     * @param null|mixed $division_id
     *
     * @return null|false|mixed|string
     */
    public function getCustomizeOption($name, $default = '', $division_id = null)
    {
        $option = OptionHelper::get($name, []);
        $division_id = $this->getURLParam('divisionId', $division_id);

        if (!empty($division_id) && count($option) === 0) {
            $name = "division_{$division_id}_customer_booking_customizer_options";
            $option = OptionHelper::get($name, []);
        }

        return $option === [] ? $default : $option;
    }

    /**
     * @param       $step_templates Template[]
     * @param mixed $params
     * @param mixed $filter_names
     *
     * @return string
     */
    public function renderSteps($step_templates, $params = [], &$filter_names = [])
    {
        $html = '';
        foreach ($step_templates as $index => $step) {
            $filter_names[] = $filter_name = StringHelper::getPrefixed(
                'booking_form_step_' . $step->getName() . '_html',
                'machine'
            );
            $html .= apply_filters($filter_name, $step->get($params), $this->shortcode_id, $index);
        }

        $html = apply_filters(
            StringHelper::getPrefixed('booking_form_steps_html'),
            $html,
            $this->shortcode_id
        );

        return $html;
    }

    public function getWorkGuaranteeTemplates(): array
    {
        $vars = [];
        foreach ($this->work_guarantee_templates as $name => $template_name) {
            $vars[$name] = new Template($template_name);
        }

        return $vars;
    }

    /**
     * @param mixed $selected
     *
     * @return Template[]
     */
    public function loadSteps($selected = [])
    {
        $this->step_templates = apply_filters(
            StringHelper::getPrefixed('booking_form_steps'),
            $this->step_templates,
            $this->shortcode_id
        );

        // filter selected steps
        $step_templates = array_filter($this->step_templates, function ($key) use ($selected) {
            return in_array($key, $selected);
        }, ARRAY_FILTER_USE_KEY);

        return array_map(function ($template_name) {
            return new Template($template_name);
        }, array_values($step_templates));
    }

    public function getLogoUrl(): string
    {
        $logo = '';

        if (\has_custom_logo()) {
            $custom_logo_id = get_theme_mod('custom_logo');
            $image = wp_get_attachment_image_src($custom_logo_id, 'full');
            $logo = !empty($image[0])
                ? $image[0]
                : '';
        }

        if (empty($logo)) {
            $logo = FSHelper::url('build/img/graphics/jims-logo.svg');
        }

        return $logo;
    }
}
