<?php

namespace JimsCustomerBooking;

use JimsCustomerBooking\Helpers\StringHelper;

if (!class_exists('JimsCustomerBooking\Template')) {
    class Template
    {
        /**
         * @var string
         */
        protected $base_folder = JIMS_CBP_DIR . '/includes/templates/';

        /**
         * @var string
         */
        protected $subfolder = '';

        /**
         * @var string
         */
        protected $default_template = 'default';

        /**
         * @var string
         */
        protected $suffix = 'tpl.php';

        /**
         * @var string
         */
        protected $name = '';

        public function __construct(string $template_name)
        {
            if ($this->templateExists($template_name)) {
                $this->name = $template_name;
            } else {
                $this->name = $this->default_template;
            }
        }

        public function templateExists(string $template_name)
        {
            return \file_exists($this->getTemplatePath($template_name));
        }

        /**
         * @return string
         */
        public function getTemplatePath(string $template_name)
        {
            return sprintf('%s%s%s.%s', $this->base_folder, $this->subfolder, $template_name, $this->suffix);
        }

        /**
         * @param mixed $with_placeholders
         * @param mixed $suffix_html
         */
        public function render(array $data = [], $with_placeholders = false, $suffix_html = '')
        {
            extract($data);
            $closing_html = '';

            if (!empty($suffix_html)) {
                $closing_html = $suffix_html;
            }

            if ($with_placeholders) {
                $html = $this->get($data);
                echo $html . $closing_html;

                return;
            }

            include $this->getTemplatePath($this->name);
            echo $closing_html;
        }

        public function get(array $data = [])
        {
            extract($data);
            ob_start();

            include $this->getTemplatePath($this->name);
            $output = ob_get_clean();

            $html = apply_filters(StringHelper::getPrefixed('template_output', 'machine'), $output, $this->name, $data);

            return $this->replacePlaceholders($html, $data);
        }

        public function getName(): string
        {
            return $this->name;
        }

        /**
         * @param mixed $html
         * @param mixed $data
         *
         * @return array|string|string[]
         */
        protected function replacePlaceholders($html, $data)
        {
            // remove arrays from data
            $data = array_filter($data, function ($value) {
                return !is_array($value) && !is_object($value);
            });

            $placeholders = array_map(function ($key) {
                return sprintf('{{%s}}', $key);
            }, array_keys($data));

            return str_replace($placeholders, array_values($data), $html);
        }
    }
}
