<?php

use JimsCustomerBooking\Template;

// phpcs:disable Generic.Files.LineLength

/**
 * @var string     $division_id
 * @var array      $service_ids
 * @var string     $logo_url
 * @var string     $hide_logo
 * @var string     $class
 * @var string     $phone_number
 * @var string     $show_country
 * @var string     $show_info_text
 * @var string     $address_text
 * @var string     $details_text
 * @var string     $description_text
 * @var array      $required_fields
 * @var string     $country
 * @var int|string $confirm_address
 */
$texts = [
    'address_text' => !empty($address_text) ? $address_text : 'The location of the work to be done or your location. We will find a Jim’s expert that is local to you',
    'details_text' => !empty($details_text) ? $details_text : 'Your Details',
    'description_text' => !empty($description_text) ? $description_text : 'Tell us a little more about the work',
];

$customCheckbox = new Template('partial.custom-checkbox');

?><div class="step client-info nflm" data-name="info" hidden>
    <h2 class="step__title fs-larger color-themed sr-only js-step-title" tabindex="-1">Book a service:</h2>

    <h3 class="sr-only">Selected services</h3>
    <ul class="selected-services"></ul>
    <fieldset class="location-details">
        <legend class="color-themed <?= !$show_info_text ? 'sr-only' : ''; ?>">
            <?= $texts['address_text']; ?>
        </legend>
        <div class="location-details__outer">
            <?php if ($show_country) { ?>
                <div>
                    <label class="sr-only" for="country">Country</label>
                    <div class="custom-select">
                        <select id="country" name="country" autocomplete="off" required></select>
                    </div>
                </div>
            <?php } else { ?>
                <input id="country" type="hidden" name="country" value="<?= $country; ?>">
            <?php } ?>

            <div class="location-details__address">
                <div class="has-icon is-round-icon is-background" data-country="<?= strtolower($country); ?>">
                    <label class="sr-only" for="address">Address</label>
                    <input
                        type="text"
                        id="address"
                        name="address"
                        placeholder="Address *"
                        autocomplete="off"
                        required
                        data-error-message="Please provide a valid address for the work to be completed at"
                        aria-describedby="address-error">
                    <div class="jims-autocomplete"></div>
                </div>
                <div id="address-error" class="address-error-message js-address-error-message" hidden></div>
            </div>

            <?php if ((int) $confirm_address === 1) { ?>
                <div class="location-details__confirm js-location-confirm">
                    <div class="location-details__confirm-outer">
                        <?= $customCheckbox->render([
                            'name' => 'location_confirm',
                            'label' => "I have checked the address above and confirm it's correct.",
                            'required' => true,
                        ], true); ?>
                    </div>
                </div>
            <?php } elseif ($confirm_address === 'popup') {
                ?><input type="checkbox" name="location_confirm" hidden required><?php
            } ?>
        </div>
    </fieldset>

    <fieldset class="contact-details">
        <legend class="color-themed <?= !$show_info_text ? 'sr-only' : ''; ?>">
            <?= $texts['details_text']; ?>
        </legend>

        <div class="contact-details__outer">
            <div class="has-icon is-person">
                <label class="sr-only" for="firstname">First Name</label>
                <input
                    type="text"
                    name="firstname"
                    id="firstname"
                    data-error-message="Please provide your first name"
                    placeholder="First Name <?= in_array('firstname', $required_fields) ? '*' : ''; ?>"
                    <?= in_array('firstname', $required_fields) ? 'required' : ''; ?>>
            </div>
            <div class="has-icon is-person">
                <label class="sr-only" for="lastname">Last Name</label>
                <input
                    type="text"
                    name="lastname"
                    id="lastname"
                    data-error-message="Please provide your last name"
                    placeholder="Last Name <?= in_array('lastname', $required_fields) ? '*' : ''; ?>"
                    <?= in_array('lastname', $required_fields) ? 'required' : ''; ?>>
            </div>
            <div class="has-icon is-email">
                <label class="sr-only" for="email">Email</label>
                <input
                    type="email"
                    name="email"
                    id="email"
                    data-error-message="Please provide your email address"
                    placeholder="Email <?= in_array('email', $required_fields) ? '*' : ''; ?>"
                    <?= in_array('email', $required_fields) ? 'required' : ''; ?>>
            </div>
            <div class="has-icon is-phone">
                <label class="sr-only" for="phone">Phone</label>
                <input
                    type="tel"
                    name="phone"
                    data-error-message="Please provide your phone number"
                    id="phone"
                    placeholder="Phone <?= in_array('phone', $required_fields) ? '*' : ''; ?>"
                    <?= in_array('phone', $required_fields) ? 'required' : ''; ?>>
            </div>
            <div class="contact-details__field is-business has-icon">
                <label class="sr-only" for="businessName">Business name</label>
                <input
                    type="text"
                    name="businessName"
                    id="businessName"
                    placeholder="Business Name <?= in_array('businessName', $required_fields) ? '*' : ''; ?>"
                    <?= in_array('businessName', $required_fields) ? 'required' : ''; ?>>
            </div>
        </div>
    </fieldset>

    <fieldset>
        <legend class="color-themed <?= !$show_info_text ? 'sr-only' : ''; ?>">
            <label for="comments">
                <?= $texts['description_text']; ?>
            </label>
        </legend>

        <textarea
            name="comments"
            rows="3"
            id="comments"
            placeholder="Give us a general idea <?= in_array('comments', $required_fields) ? '*' : ''; ?>"
            <?= in_array('comments', $required_fields) ? 'required' : ''; ?>></textarea>
    </fieldset>

    <fieldset>
        <legend class="color-themed sr-only">Would you like to subscribe for special offers from Jim's Group?</legend>
            <?= $customCheckbox->render([
                'name' => 'subscribed',
                'label' => '{{subscribe_text}}',
                'accessibleLabel' => 'Yes',
            ], true); ?>
    </fieldset>
</div>
