<?php

use JimsCustomerBooking\Helpers\StringHelper;
use JimsCustomerBooking\Template;
use JimsCustomerBooking\Helpers\FSHelper;

// phpcs:disable Generic.Files.LineLength

/**
 * @var string $css_class
 * @var string $title
 * @var array  $options
 * @var array  $pages
 * @var array  $parts
 */
$fms_enviroment = [
    'production' => 'Production',
    'development' => 'Development',
    'uat' => 'UAT',
    'hf' => 'Hotfix',
    'staging' => 'Staging',
    'int' => 'Integration',
    //    'hf2' => 'Hotfix2',
];

$hostname = $_SERVER['HTTP_HOST'];
if (is_admin() && in_array($hostname, ['localhost', 'jims-wp-default.local', '127.0.0.1'])) {
    $fms_enviroment['local'] = 'Local';
}

/**
 * @var Template $header
 * @var Template $footer
 */
$header = $footer = null;

$header = $parts['header'];
$footer = $parts['footer'];
// extract ($parts);

$info_fields = [
    'firstname',
    'lastname',
    'email',
    'phone',
    'businessName',
    'comments',
];

$default_required_info_fields = array_filter($info_fields, function ($field) use ($options) {
    $default_required_info_fields = [
        'firstname',
        'lastname',
        'email',
        'phone',
    ];

    $in_database = isset($options['required_fields-' . StringHelper::slugify($field)]);
    $in_default = in_array($field, $default_required_info_fields) && !$in_database;

    return $in_database || $in_default;
});

$domain_name = $_SERVER['HTTP_HOST'];

// Checking if the domain is not jims not avoid security issue of compromised domain
if (!str_starts_with($domain_name, 'jims')) {
    $domain_name = 'localhost';
}

?>
<script>
    window.wpcbSecret = '<?= 'admin_' . md5('admin'); ?>';
</script>

<?php if (!empty($header)) { ?>
    <?php $header->render([
        'title' => 'Settings',
        'description' => 'This page, allows to modify the behavior of Jim\'s customer booking form (native)',
        'pages' => $pages,
        'url' => get_template_directory_uri(),
    ], true); ?>
<?php } ?>

<div class="<?= $css_class; ?>">
    <section class="jims-main">
        <form id="jims-events-form">
            <div class="loader">
                <div class="loader-icon">
                    <img src="<?= FSHelper::url('build/img/graphics/loader.gif'); ?>" alt="loading">
                </div>
            </div>
            <div class="settings-text settings-text--blue">
                <ul>
                    <li>If this page doesn't work, please contact support and ask to whitelist your server's IP.
                    </li>
                </ul>
            </div>

            <?php // Basic settings ?>
            <fieldset class="setting-group basic nflm collapsable-fieldset is-open">
                <legend class="collapsable-fieldset__control">Basic Settings</legend>
                <div class="inner collapsable-fieldset__content">
                    <label for="divisions">Choose a Division</label>
                    <select id="divisions" name="bookingFormDivisionId" class="widefat">
                        <option>No Division Available</option>
                    </select>
                    <small>If this option doesn't work, please contact infotech@jims.net and ask to whitelist your
                        server's IP</small>

                    <label for="country">Country</label>
                    <select class="widefat" name="country" id="country">
                       <option>No Country Available</option>
                    </select>

                    <label for="phone">Phone</label>
                    <input type="text" class="widefat" id="phone" name="phone"
                           value="<?= isset($options['phone']) ? $options['phone'] : ''; ?>">


                    <!-- Confirm Address Option -->
                    <label for="confirm-address">Show User Address Confirmation?</label>
                    <select name="confirmAddress" id="confirm-address" class="widefat">
                        <option value="0" <?= isset($options['confirmAddress']) && $options['confirmAddress'] == 0 || !isset($options['confirmAddress']) ? 'selected' : ''; ?>>No</option>
                        <option value="1" <?= (isset($options['confirmAddress']) && $options['confirmAddress'] == 1) ? 'selected' : ''; ?>>Checkbox</option>
                        <option value="popup" <?= (isset($options['confirmAddress']) && $options['confirmAddress'] == 'popup') ? 'selected' : ''; ?>>Popup</option>
                    </select>

                    <div <?= is_admin() && isset($_GET['debug_mode']) ? '' : 'style="display:none;"'; ?>">
                        <!-- Enable use in iframes Option -->
                        <label for="use-in-iframe" >Enable use in iframes</label>
                        <select name="iframesAllowed" id="use-in-iframe" class="widefat">
                            <option value="0" <?= isset($options['iframesAllowed']) && $options['iframesAllowed'] == 0 || !isset($options['iframesAllowed']) ? 'selected' : ''; ?>>No</option>
                            <option value="1" <?= (isset($options['iframesAllowed']) && $options['iframesAllowed'] == 1) ? 'selected' : ''; ?>>Yes</option>
                        </select>
                        <small>Enabling for iframes is not recommended.</small>
                    </div>

                    <label for="debugMode">Enable Debug Mode</label>
                    <select name="debugMode" id="debug_mode" class="widefat">
                        <option value="0" <?= $options['debugMode'] == 0 ? 'selected' : ''; ?>>No</option>
                        <option value="1" <?= $options['debugMode'] == 1 ? 'selected' : ''; ?>>Yes</option>
                    </select>
                    <small>Enable it if this site is staging.</small>
                </div>
            </fieldset>

            <?php // Settings group for styling ?>
            <fieldset class="setting-group style nflm collapsable-fieldset">
                <legend class="collapsable-fieldset__control">Styling</legend>
                <div class="inner collapsable-fieldset__content">
                    <label for="css">Form Main Color</label>
                    <input
                        name="bookingFormMainColor"
                        type="color"
                        value="<?= isset($options['bookingFormMainColor']) ? $options['bookingFormMainColor'] : '#0b75b4'; ?>">
                </div>
            </fieldset>

            <?php // Address Autocomplete & Validation Settings ?>
            <fieldset class="setting-group auto-complete collapsable-fieldset is-open">
                <legend class="collapsable-fieldset__control">Address Autocomplete & Validation Settings</legend>
                <div class="inner collapsable-fieldset__content">
                    <div class="settings-text">WPCB plugin offers two options for address autocompletion and validation:
                        <ul>
                            <li>
                                Jim's: This option is free of charge but may have slower performance compared to Google Autocomplete (default setting).
                            </li>
                            <li>
                                Google: This option provides superior performance, but requires adding a credit card to obtain an API key.
                            </li>
                        </ul>
                    </div>

                    <!-- Select if the user wants to use Google Auto Complete -->
                    <label for="use-google-autocomplete">Address service</label>
                    <select name="use_google_autocomplete" id="use-google-autocomplete" class="widefat">
                        <option value="0" <?= $options['use_google_autocomplete'] == 0 ? 'selected' : ''; ?>>
                            Jim's
                        </option>
                        <option value="1" <?= $options['use_google_autocomplete'] == 1 ? 'selected' : ''; ?>>
                            Google
                        </option>
                    </select>

                    <!-- Google API Key -->
                    <label for="google-api-key">Google API Key - Required for Google service</label>
                    <input type="password" class="widefat" id="google-api-key" name="google_api_key"
                           value="<?= isset($options['google_api_key']) ? $options['google_api_key'] : ''; ?>">

                </div>
            </fieldset>

            <?php // Form Labels (restricted access) ?>
            <fieldset class="setting-group form-text <?= is_admin() && isset($_GET['debug_mode']) ? '' : 'hidden'; ?> collapsable-fieldset">
                <!-- Form Text Settings -->
                <legend class="collapsable-fieldset__control">Form Labels (restricted access)</legend>
                <div class="inner collapsable-fieldset__content">
                    <div class="settings-text settings-text-warning">
                        Please, to avoid TOS violation, before any changes in this section, notify the <a
                                href="mailto:infotech@jims.net">Infotech</a>.
                    </div>
                    <label for="form-title">Address Text</label>
                    <input type="text" class="widefat" id="form-address-text" name="bookingFormAddressText"
                           value="<?= isset($options['bookingFormAddressText']) ? $options['bookingFormAddressText'] : ''; ?>">

                    <label for="form-subtitle">Details Text</label>
                    <input type="text" class="widefat" id="form-details-text" name="bookingFormDetailsText"
                           value="<?= isset($options['bookingFormDetailsText']) ? $options['bookingFormDetailsText'] : ''; ?>">

                    <label for="form-button">Description Text</label>
                    <input type="text" class="widefat" id="form-button" name="bookingFormDescriptionText"
                           value="<?= isset($options['bookingFormDescriptionText']) ? $options['bookingFormDescriptionText'] : ''; ?>">
                </div>
            </fieldset>

            <?php // Form Required Fields (restricted access) ?>
            <fieldset class="setting-group form-required-fields collapsable-fieldset <?= is_admin() && isset($_GET['debug_mode']) ? '' : 'hidden'; ?>">
                <!-- Form Text Settings -->
                <legend class="collapsable-fieldset__control">Form Required Fields (restricted access)</legend>
                <div class="inner collapsable-fieldset__content">
                    <div class="settings-text settings-text-warning">
                        Please, to avoid TOS violation, before any changes in this section, notify the <a href="mailto:infotech@jims.net">Infotech</a>.
                    </div>

                    <!-- Multiselect for required fields -->
                    <label for="required-fields">Required Fields</label>
                    <?php foreach ($info_fields as $field) {
                        $cf = StringHelper::slugify($field); ?>
                        <p class="checkbox-field">
                            <label style="display: inline"
                                   for="<?= $cf; ?>"><?= ucwords($field); ?></label>
                            <input type="checkbox" id="<?= $cf; ?>" name="required_fields-<?= $cf; ?>"
                                   value="<?= $field; ?>" <?= in_array($field, $default_required_info_fields) ? 'checked' : ''; ?>>
                        </p>
                    <?php } ?>
                </div>
            </fieldset>

            <fieldset class="setting-group email-settings collapsable-fieldset <?= is_admin() && isset($_GET['debug_mode']) ? '' : 'hidden'; ?>">
                <!-- Email Settings -->
                <legend class="collapsable-fieldset__control">Email Settings (restricted access)</legend>
                <div class="inner collapsable-fieldset__content">
                    <div class="settings-text settings-text-warning">
                        This section should be handled by the Infotech team. Please, try to avoid changes in this.
                    </div>

                    <label for="enable-notifications">Enable Notifications</label>
                    <select name="enable_notifications" id="enable-notifications" class="widefat">
                        <option value="0" <?= $options['enable_notifications'] == 0 ? 'selected' : ''; ?>>No
                        </option>
                        <option value="1" <?= $options['enable_notifications'] == 1 ? 'selected' : ''; ?>>Yes
                        </option>
                    </select>

                    <label for="notification-scheduler">Notification Scheduler</label>
                    <select name="notification_scheduler" id="notification-scheduler" class="widefat">
                        <option value="0" <?= $options['notification_scheduler'] == 0 ? 'selected' : ''; ?>>
                            System (linux cron)
                        </option>
                        <option value="1" <?= $options['notification_scheduler'] == 1 ? 'selected' : ''; ?>>
                            Wordpress Cron
                        </option>
                    </select>
                    <small>Choose the scheduler to use for sending notifications. The default should be System, however
                        if you don't have access to the website administration panel to setup a system cron. You can set
                        the Wordpress scheduler. However Wordpress scheduler it depends from user traffic, so it's not
                        accurate.</small>


                    <div class="scheduler-data smtp-data <?= isset($options['notification_scheduler']) && $options['notification_scheduler'] == 0 ? 'hidden' : ''; ?>">

                        <label for="from-email">From Email</label>
                        <input type="text" class="widefat" id="from-email" name="from_email"
                               value="<?= isset($options['from_email']) ? $options['from_email'] : implode('@', ['admin', $domain_name]); ?>">

                        <label for="email-subject">Email Subject</label>
                        <input type="text" class="widefat" id="email-subject" name="email_subject"
                               value="<?= isset($options['email_subject']) ? $options['email_subject'] : 'CB3 Notification'; ?>">

                        <label for="smtp-server">SMTP Server</label>
                        <input type="text" class="widefat" id="smtp-server" name="smtp_server"
                               value="<?= isset($options['smtp_server']) ? $options['smtp_server'] : 'smtp.gmail.com'; ?>">

                        <label for="smtp-port">SMTP Port</label>
                        <input type="text" class="widefat" id="smtp-port" name="smtp_port"
                               value="<?= isset($options['smtp_port']) ? $options['smtp_port'] : '587'; ?>">

                        <label for="smtp-username">SMTP Username</label>
                        <input type="text" class="widefat" id="smtp-username" name="smtp_username"
                               value="<?= isset($options['smtp_username']) ? $options['smtp_username'] : ''; ?>">

                        <label for="smtp-password">SMTP Password</label>
                        <input type="password" class="widefat" id="smtp-password" name="smtp_password"
                               value="<?= isset($options['smtp_password']) ? $options['smtp_password'] : ''; ?>">

                    </div>

                    <div class="scheduler-data scheduler-command <?= isset($options['notification_scheduler']) && $options['notification_scheduler'] == 1 ? 'hidden' : ''; ?>">
                        <pre>Some command</pre>
                    </div>
                </div>
            </fieldset>

            <?php // Redirection Settings ?>
            <fieldset class="setting-group redirection collapsable-fieldset">
                <legend class="collapsable-fieldset__control">Redirection Settings</legend>
                <div class="inner collapsable-fieldset__content">
                    <label for="user-redirect">Set Redirect if you want to redirect after forms successful submission
                        (thank
                        you
                        page)</label>

                    <select name="use_redirect" id="use-redirect" class="widefat">
                        <option value="0" <?= $options['use_redirect'] == 0 ? 'selected' : ''; ?>>No</option>
                        <option value="1" <?= $options['use_redirect'] == 1 ? 'selected' : ''; ?>>Yes</option>
                    </select>


                    <label for="redirect-success-url">Set the URL you want the form to redirect after successful
                        submission</label>
                    <input type="text" class="widefat" id="redirect-success-url" name="redirect_success_url"
                           value="<?= isset($options['redirect_success_url']) ? $options['redirect_success_url'] : ''; ?>">
                </div>
            </fieldset>

            <?php // Settings group for compatibility ?>
            <fieldset class="setting-group compatibility collapsable-fieldset">
                <legend class="collapsable-fieldset__control">Compatibility Settings</legend>
                <div class="inner collapsable-fieldset__content">
                    <div class="settings-text settings-text-warning">
                        Please, before doing any changes here, make sure you know what are you doing. This setting sets
                        the HTTP version on cUrl requests. If you having FMS connectivity issues and you don't know what
                        this is, please contact the <a
                                href="mailto:infotech@jims.net">Infotech</a>..
                    </div>

                    <label for="css">HTTP Version (for FMS communications)</label>
                    <select name="httpVersionForFMS" id="http-version" class="widefat">
                        <option value="0" <?= $options['httpVersionForFMS'] == 0 ? 'selected' : ''; ?>>Default
                        </option>
                        <option value="1" <?= $options['httpVersionForFMS'] == 1 ? 'selected' : ''; ?>>HTTP 1.1
                        </option>
                    </select>

                    <label for="css">HTTP Version (for Internal communications)</label>
                    <select name="httpVersionForInternal" id="http-version" class="widefat">
                        <option value="0" <?= $options['httpVersionForInternal'] == 0 ? 'selected' : ''; ?>>
                            Default
                        </option>
                        <option value="1" <?= $options['httpVersionForInternal'] == 1 ? 'selected' : ''; ?>>HTTP
                            1.1
                        </option>
                    </select>
                </div>
            </fieldset>

            <?php // FMS Communication Settings (restricted access) ?>
            <fieldset class="setting-group fms-communication-settings collapsable-fieldset <?= is_admin() && isset($_GET['debug_mode']) ? '' : 'hidden'; ?>">
                <legend class="collapsable-fieldset__control">FMS Communication Settings (restricted access)</legend>
                <div class="inner collapsable-fieldset__content">
                    <div class="settings-text">
                        <ul>
                            <li>These settings are used to communicate with the FMS API.</li>
                        </ul>
                    </div>

                    <label for="replace-nz-special">Replace Special Characters For New Zealand Leads</label>
                    <select name="replace_nz_special" id="replace-nz-special" class="widefat">
                        <option value="0" <?= $options['replace_nz_special'] == 0 ? 'selected' : ''; ?>>No
                        </option>
                        <option value="1" <?= $options['replace_nz_special'] == 1 ? 'selected' : ''; ?>>Yes
                        </option>
                    </select>
                </div>
            </fieldset>

            <?php // FMS Target Environment ?>
            <fieldset class="setting-group target-environment collapsable-fieldset <?= is_admin() && isset($_GET['debug_mode']) ? '' : 'hidden'; ?>">
                <legend class="collapsable-fieldset__control">FMS Target Environment (restricted access)</legend>
                <div class="inner collapsable-fieldset__content">
                    <div class="settings-text">
                        Please note that some of these environments are only available to Jim's Group staff and they
                        require
                        network authentication.
                    </div>
                    <select name="fms_environment" id="fms-environment" class="widefat">
                        <?php foreach ($fms_enviroment as $key => $value) { ?>
                            <option value="<?= $key; ?>" <?= $options['fms_environment'] == $key ? 'selected' : ''; ?>><?= $value; ?></option>
                        <?php } ?>
                    </select>
                </div>
            </fieldset>

            <div class="button-wrapper">
                <input id="save-form" class="button button-primary" type="submit" value="Save Values">

                <!-- Clear Form Button -->
                <!--                <input id="clear-form" type="reset" class="button button-warning" value="Clear Form">-->

                <!-- Delete Form Button -->
                <input id="delete-form" type="button" class="button button-danger" value="Delete Values">
            </div>
        </form>
    </section>
</div>
