<?php

use JimsCustomerBooking\Helpers\FSHelper;
use JimsCustomerBooking\Helpers\OptionHelper;
use JimsCustomerBooking\Template;

// phpcs:disable Generic.Files.LineLength

/**
 * @var string     $prefix_class
 * @var string     $division_id
 * @var array      $service_ids
 * @var string     $logo_url
 * @var string     $hide_logo
 * @var string     $class
 * @var string     $phone_number
 * @var string     $show_country
 * @var string     $show_info_text
 * @var string     $enable_popup
 * @var string     $grouping_enabled
 * @var string     $custom_order_enabled
 * @var string     $alphabetical_order
 * @var array      $show_only_services
 * @var array      $show_only_divisions
 * @var array      $exclude_services
 * @var string     $steps
 * @var int        $steps_count
 * @var string     $service_list
 * @var string     $country
 * @var string     $button_text
 * @var int|string $confirm_address
 * @var string     $iframes_allowed
 * @var string     $component_id
 * @var string     $work_guarantee_button
 * @var string     $work_guarantee_dialog
 */
$args = array_merge(
    OptionHelper::getGlobalSettings(),
    [
        'show_step_headings' => $data['show_step_headings'],
        'hide_step_search' => $data['hide_step_search'],
        'bookingFormDivisionId' => $division_id,
        'country' => $country,
        'bookingFormServiceIds' => array_map(fn ($service_id) => (int) $service_id, $service_ids),
        'show_country' => boolval($show_country),
        'show_steps' => explode(',', $show_steps),
        'enable_popup' => boolval($enable_popup),
        'address_confirmation_enabled' => $confirm_address,
        'iframes_allowed' => boolval($iframes_allowed),
        'grouping_enabled' => boolval($grouping_enabled),
        'custom_order_enabled' => $custom_order_enabled,
        'alphabetical_order' => boolval($alphabetical_order),
        'show_only_services' => array_map(fn ($service_id) => (int) $service_id, $show_only_services),
        'show_only_divisions' => array_map(fn ($division_id) => (int) $division_id, $show_only_divisions),
        'exclude_services' => array_map(fn ($service_id) => (int) $service_id, $exclude_services),
        'bookingFormServiceList' => $service_list, // the service list is a string used on FMS to identify the custom service list
    ],
);

// because there can only be one template for a component (techically), we need
// to create different templates for each instance of CB and assign them a
// unique ID that we use to load the right template for each instance. We would
// need to rewrite a lot of CB code to use a generic template so this is a
// middleground until we do that.
$uniqueID = uniqid('wpcb-instance-');

// We need to pull the css versio number for cache busting
$frontAsset = FSHelper::require('build/front.asset.php');

$loader = new Template('partial.loader');
$header = new Template('partial.header');
$footer = new Template('partial.footer');
$offline = new Template('partial.offline');

// $rootEl = boolval($enable_popup) ? 'dialog' : 'div';
$rootEl = 'div';

$componentClasses = [];
if(boolval($enable_popup)) {
    $componentClasses[] = 'is-popup';
}

?><wpcb-instance
    class="<?= implode(' ', $componentClasses); ?>"
    data-template="<?= $uniqueID; ?>"
    data-settings="<?= esc_attr(json_encode($args)); ?>">
</wpcb-instance>

<template id="<?= $uniqueID; ?>">
    <style>
        .wpcb {
            --wpcb-graphic-confetti: url('<?= FSHelper::url('build/img/graphics/confetti.gif'); ?>');
        }
    </style>
    <?php if (!empty($args['debugMode']) && $args['debugMode'] === true) { ?>
        <link rel="stylesheet" href="<?= FSHelper::url('build/front.css'); ?>?ver=<?= $frontAsset['version']; ?>">
    <?php } else { ?>
        <style><?= FSHelper::content('build/front.css'); ?></style>
    <?php } ?>

    <<?= $rootEl; ?> class="{{class}}">
        <div class="wpcb__outer">
            <?= $header->render($data, true); ?>

            <!-- FORM  -->
            {{work_guarantee_button}}

            <form class="form js-form">
                <div class="form__inner js-form-inner">
                    <div class="notifications nflm" aria-hidden="true"></div>
                    {{steps}}
                </div>

                <?= $footer->render($data, true); ?>
                <?= $loader->render($data, true); ?>
                <?= $offline->render($data, true); ?>
            </form>
        </div>


        {{work_guarantee_dialog}}

        <?php if ($confirm_address === 'popup') {
            $validationPopup = new Template('partial.address-validation-dialog');
            echo $validationPopup->render();
        } ?>

        <div class="js-aria-live sr-only" aria-live=assertive aria-atomic=true></div>

    </<?= $rootEl; ?>>
</template>
