<?php

// phpcs:disable Generic.Files.LineLength

/*
 * @var string $title
 * @var array $fields
 * @var array $services
 * @var string $css_class
 * @var \JimsCustomerBooking\JimsOptions $JimsOptions
 * @var $selected_division int
 * @var $pages     array
 * @var $parts     array
 * @var $native_fields array
 */

use JimsCustomerBooking\Helpers\StringHelper;
use JimsCustomerBooking\Template;
use JimsCustomerBooking\Helpers\FSHelper;

/**
 * @var Template $header
 * @var Template $footer
 */
$header = $footer = null;
extract($parts);
?>

<?php if (!empty($header)) { ?>
    <?php $header->render([
        'title' => 'Shortcode Generator',
        'description' => 'Welcome to the Customer Booking (CB) Shortcode Generator! This intuitive form allows you to easily create a custom shortcode for embedding a dynamic booking form anywhere on your WordPress site. Whether you\'re looking to add a booking form to a page, post, or as a popup window, our generator streamlines the process, ensuring you have a tailored solution that meets your specific requirements.',
        'pages' => $pages,
        'url' => get_template_directory_uri(),
    ], true); ?>
<?php } ?>

<div class="<?= $css_class; ?>">

    <section class="cb3">
        <div class="how-it-works">
            <h2><label for="how-it-works">How it Works<span style="display:inline-block;width: 1em; fill: #0d66c2; margin-left: 10px; vertical-align: bottom;"><?= FSHelper::content('build/img/icons/question-circle.svg'); ?></span></label></h2>
            <input type="checkbox" id="how-it-works" style="display:none;">
            <div class="inner">
                <ul>
                    <li><strong>Generate Your Shortcode:</strong> Once you've configured the options to your liking,
                        simply click the "Generate Shortcode" button located at the bottom of the form. Our generator
                        will instantly create a custom shortcode based on your specified options.
                    </li>

                    <li><strong>Copy and Use Your Shortcode:</strong> Directly below the generator button, you'll find
                        the area displaying your calculated shortcode. Copy this shortcode and paste it into any page,
                        post, or widget area where you want your Customer Booking form to appear.
                    </li>
                </ul>

                <p>For more information on how to use the shortcode, please watch the video <a href="https://jimschubert.github.io/customer-booking-plugin-docs/docs/shortcodes/booking-form/" target="_blank">Jim's Resources: How to use the CB3 shortcode generator</a>.</p>
            </div>
        </div>

        <form name="native-booking-form-shortcode-generator">
            <input type="hidden" id="shortcode-name"
                   value="<?= StringHelper::getPrefixed('booking_form', 'machine'); ?>">

            <!--- Radio Buttons -->
            <div class="form-group radios">
                <h2>Form Section Visibility Settings</h2>

                <div class="radio-group">
                    <!--- Popup Buttons -->
                    <wpcb-toggle class="use-popup" name="enable_popup" text="Use CB as popup"></wpcb-toggle>

                    <!-- Enable / Disable Country Field -->
                    <wpcb-toggle class="enable-country" name="show_country"
                                 text="Disable Country Selection"></wpcb-toggle>

                    <!-- Show / Hide Form Texts -->
                    <wpcb-toggle class="hide-info-texts" name="show_info_text"
                                 text="Hide fields info texts"></wpcb-toggle>

                    <!-- Use Contained / Headless Form -->
                    <wpcb-toggle class="hide-header" name="contained" text="Hide Header"></wpcb-toggle>

                    <wpcb-toggle name="show_step_headings" text="Show step headings"></wpcb-toggle>
                    <wpcb-toggle name="hide_step_search" text="Hide step search"></wpcb-toggle>
                </div>
            </div>

            <div class="form-group logo">
                <h2>Logo Settings</h2>
                <!-- Set Logo Image -->
                <label for="logo-image">Set Logo URL</label>
                <input type="url" name="logo_url" id="logo-url">

                <!-- Set Logo Visibility if contained isn't active -->
                <wpcb-toggle class="hide-logo" name="hide_logo" text="Hide Logo"></wpcb-toggle>
            </div>
            <p class="form-group__disabled-info">This section is available only if the header is visible.</p>

            <!--- Allow URL Params (only for admins), this enables extra fields-->
            <div class="form-group url-params">
                <h2>URL Params Settings</h2>
                <wpcb-notification>
                    URL Params features allows the form to be pre-filled with data from the URL. This is useful for
                    pre-filling the form with data from other systems.
                </wpcb-notification>
                <wpcb-toggle for="url-params" name="accept_url_params" class="allow-url-params"
                             text="Allow URL Params"></wpcb-toggle>
                <small>Available Url Params to include</small>
                <select class="url-params" id="url-params" name="allowed_url_params" multiple>
                    <option value="divisionId">Division Id</option>
                    <option value="countryCode">Country Code</option>
                </select>
            </div>

            <!--- Select steps  (only for admins) -->
            <div class="form-group steps">
                <h2>Form Steps</h2>
                <wpcb-notification>
                    This section allows to select the extra steps that will be displayed in the form. For safety reasons
                    the services and client info steps are always displayed.
                </wpcb-notification>
                <select class="steps" name="show_steps" multiple>
                    <option value="divisions">Divisions</option>
                </select>
            </div>

            <div class="form-group divisions">
                <h2>Division Settings</h2>
                <wpcb-notification>
                    This section enables the selection of the division to be displayed in the form. If no division is
                    chosen, the form will show all divisions by default. The selected division defaults to the one
                    specified on the main settings page.
                </wpcb-notification>

                <label for="division-id">Select a different division for the shortcode</label>
                <select class="division-id" id="division-id" name="division_id">
                </select>

                <label for="show-divisions-only">Show only specific divisions on Division Select</label>
                <select class="show-divisions-only" id="show-divisions-only" name="show_only_divisions" multiple disabled></select>
            </div>

            <div class="form-group services">
                <h2>Service Settings</h2>
                <wpcb-notification>
                    This section enables you to manage the services displayed in the form. You have the option to
                    preselect services, display only certain services, or utilize a custom service list.
                </wpcb-notification>

                <!-- show services lists -->
                <div class="service-list">

                    <!-- Service Search -->
                    <div class="service-search-container">
                        <input type="text" name="service_search" id="service-search"
                               placeholder="Search for a service name...">

                        <img src="<?= FSHelper::url('build/img/icons/search.svg'); ?>" alt="">
                    </div>

                    <!-- Service Results -->
                    <div class="service-results empty"></div>

                    <wpcb-notification type="warning">
                        If you don't know the id of the service you are looking for, you can drag and drop the service
                        from the list above to the text field you want to add it.
                    </wpcb-notification>
                </div>

                <!-- Preselect Services -->
                <wpcb-toggle for="service-ids" class="preselect-services" text="Preselect Services"
                             name="preselect_services"></wpcb-toggle>
                <input type="text" name="service_ids" id="service-ids" value="" placeholder="Service Ids">
                <small>Comma separated list of service ids. This feature skips the service select step adding the
                    selected services to the final step.</small>

                <!-- Show Only Services -->
                <wpcb-toggle for="show-only-services" class="show-only-services"
                             text="Show Only Services"></wpcb-toggle>
                <input type="text" name="show_only_services" id="show-only-services" value="" placeholder="Service Ids">
                <small>Comma separated list of service ids. This feature hides all the other services on service select
                    and shows only those selected.</small>

                <!-- Use Custom Service List -->
                <wpcb-toggle for="service-list" class="use-custom-service-list" text="Use Custom Service List"
                             name="use_custom_service_list"></wpcb-toggle>
                <input type="text" name="service_list" id="service-list" value="" placeholder="Service List Name">
                <small>Use a custom service list. This feature leverages an FMS Custom Service List, allowing you to
                    display services not included in the default list. These names can only be requested from
                    MCCC.</small>
            </div>

            <div class="form-group text-overrides">
                <h2>Text Overrides</h2>
                <wpcb-notification>
                    This section allows to override the default texts that are displayed in the form.
                </wpcb-notification>
                <!--- Text Overrides -->

                <!--                <label for="loader-search-text">Loader Search Text</label>-->
                <!--                <input type="text" name="loader_search_text" id="loader-search-text" value=""-->
                <!--                       placeholder="Searching for Franchisee in your Area...">-->
                <!--                <small>Replaces the text on the loader when searching for franchisees</small>-->

                <label for="submit-button-text">Submit Button Text</label>
                <input type="text" name="button_text" id="submit-button-text" value="" placeholder="Submit">
                <small>Replaces the text on the Submit button</small>
            </div>

            <!-- <div class="button-container">
                <button type="submit" class="button button-primary">Generate Shortcode</button>
            </div> -->
        </form>

        <p>To add the CB3 form shortcode just copy / paste the following:</p>
        <pre style="white-space: normal; word-break: break-all; outline: darkblue; background: darkcyan; color: white; border-radius: 10px; padding: 1em 0.5em;"
             id="native-booking-form-shortcode">
                <?= '[' . StringHelper::getPrefixed('booking_form', 'machine') . ']'; ?>
            </pre>
    </section>
</div>
