<?php

namespace JimsGroup\Datatypes\Franchisee;

use JimsGroup\Datatypes\Arrayable;
use JimsGroup\Datatypes\DynamicPropertiesTrait;

abstract class DataType implements Arrayable
{
    use DynamicPropertiesTrait;

    protected array $default = [];

    protected array $data = [];

    protected array $ignored = [];

    protected array $cast = [];

    /**
     * DataType constructor.
     * @throws \RuntimeException
     * @param array $data
     */
    public function __construct(array $data = [])
    {
        if (!$this->validate($data)) {
            throw new \RuntimeException('Invalid data on ' . static::class . ' object');
        }

        $this->data = array_merge($this->default, $this->data, $data);
    }

    public function __get($name)
    {
        $propertyValue = $this->getProperty($this->data, $name);
        if (!empty($propertyValue)) {
            //if the field has a cast array, initiate the class and return the object
            if (isset($this->cast[$name])) {
                $cast = $this->cast[$name];
                return new $cast($propertyValue);
            }

            return $this->data[$name];
        }

        if (method_exists($this, $name)) {
            return $this->$name();
        }

        return null;
    }

    public function __set($name, $value)
    {
        $this->data[$name] = $value;
    }

    public function __isset($name)
    {
        return isset($this->data[$name]);
    }

    public function __unset($name)
    {
        unset($this->data[$name]);
    }

    public function toArray(): array
    {
        return $this->data;
    }

    /**
     * @throws \JsonException
     */
    public function toJson(): string
    {
        return json_encode($this->data, JSON_THROW_ON_ERROR);
    }

    public function isEmpty(): bool
    {
        return empty($this->data);
    }

    /**
     * @throws \JsonException
     */
    public function __toString()
    {
        return $this->toJson();
    }

    protected function getRequiredKeysForValidation(): array
    {
        $required_keys = array_keys($this->default);

        //remove the ignored fields
        foreach ($this->ignored as $field) {
            unset($required_keys[array_search($field, $required_keys, true)]);
        }

        return $required_keys;
    }

    abstract public function validate($data): bool;
}
