<?php

namespace JimsGroup\Utilities;

/**
 * Class Logger
 * @package JimsGroup\Utilities
 */
class Logger
{
    /**
     * @var string
     */
    private static $mode = 'default'; // 'default' or 'file'


    /**
     * @var string
     */
    private static $custom_file = 'log.txt';


    /**
     * @var string
     */
    private static $root_dir = __DIR__;


    /**
     * @var float
     */
    private static $time = 0;

    /**
     * @param $message
     * @return bool
     */
    public static function log($message, $type = 'generic'): bool
    {
        if (self::$root_dir === __DIR__ && defined('ABSPATH')) {
            self::$root_dir = constant('ABSPATH');
        }

        $log = self::getLogFromMessage($message, $type);

        if (self::$mode === 'default') {
            return error_log($log);
        }

        return self::custom_log($log);
    }

    /**
     * @param $message
     * @return bool
     */
    public static function info($message, $marker = ''): bool
    {
        return self::log($message, 'info' . self::getMarker($marker));
    }

    /**
     * @param $message
     * @return bool
     */
    public static function error($message, $marker = ''): bool
    {
        return self::log($message, 'error' . self::getMarker($marker));
    }


    /**
     * @param $message
     * @return bool
     */
    public static function custom_log($message): bool
    {
        return (bool)file_put_contents(implode('/', [self::getDir(), self::$custom_file]), $message, FILE_APPEND);
    }

    /**
     * @param $message
     * @return string
     */
    public static function getLogFromMessage($message, $type = 'info'): string
    {
        $value_delimiter = ' - ';

//        if (is_array($message) || is_object($message)) {
//            $message = json_encode($message);
//        }

        $message = print_r($message, true);

        //it creates a log message with the following format:
        // phpcs:ignore
        // 2021-01-01 00:00:00 - JimsGroup\Utilities\Logger - info - This is a log message - on - /path/to/file.php - at - 123
        return implode($value_delimiter, [
            date('Y-m-d H:i:s'),
            __CLASS__,
            $type,
            $message,
            'on',
            __FILE__,
            'at',
            __LINE__
        ]) . PHP_EOL;
    }


    /**
     * @return string
     */
    public static function getDir(): string
    {
        //check env for log_dir, if not set use default
        return getenv('LOG_DIR') ?: self::$root_dir;
    }

    /**
     * @param string $stage
     * @return float
     */
    public static function timer($stage = 'start'): float
    {
        if ($stage === 'start') {
            return self::$time = microtime(true);
        } else {
            return self::$time > 0 ? microtime(true) - self::$time : 0;
        }
    }

    /**
     * @param $mode
     * @return void
     */
    public static function setMode($mode): void
    {
        self::$mode = $mode;
    }

    /**
     * @param $file
     * @return void
     */
    public static function setCustomFile($file): void
    {
        self::$custom_file = $file;
    }

    /**
     * @param string $root_dir
     * @return void
     */
    public static function setRootDir(string $root_dir): void
    {
        self::$root_dir = $root_dir;
    }

    /**
     * @param $marker
     * @return string
     */
    public static function getMarker($marker): string
    {
        return !empty($marker) ? ' ' . $marker : '';
    }
}
