<?php

namespace JimsGroup\Utilities;

/**
 * Class PerformanceTimer
 * @package JimsGroup\Utilities
 */
class PerformanceTimer
{
    /**
     * Evaluate the performance of a callback function
     * @param callable $callback
     * @param int $iterations
     * @param bool $return_average
     * @return array|float
     */
    public static function Evaluate(callable $callback, int $iterations = 1, bool $return_average = false): array|float
    {
        $times = [];
        for ($i = 0; $i < $iterations; $i++) {
            $times[] = self::Time($callback);
        }
        if ($return_average) {
            return array_sum($times) / $iterations;
        }

        return $times;
    }

    /**
     * Evaluate the performance of a callback function
     * @param callable $callback
     * @return float
     */
    public static function Time(callable $callback): float
    {
        $start = microtime(true);
        $callback();

        return microtime(true) - $start;
    }


    /**
     * Evaluate the performance of a callback function
     * @param callable $callback
     * @param int $time
     * @return mixed
     */
    public static function ProductionTime(callable $callback, float &$time): mixed
    {
        $start = microtime(true);
        $result = $callback();
        $time = microtime(true) - $start;

        return $result;
    }
}
